@@ -242,13 +242,26 @@ struct envelope {
 #define HDR_SCANNER(x)	scan822(&(x), strlen((const char *)x), 0, 0, 0, (token822 **)0)
 #endif	/* RFC976 */
 
-/* #include "libz.h" */
+struct Zpasswd {
+  /* Basic normal 'struct passwd' things */
+  const char *pw_name;
+  const char *pw_passwd;
+  uid_t pw_uid;
+  gid_t pw_gid;
+  const char *pw_gecos;
+  const char *pw_dir;
+  const char *pw_shell;
+  /* Our ``Z'' extensions */
+  long quota1, quota2;
+  const char *mail_forward;
+};
 
-#ifdef	USE_ZGETPWNAM
-#define	getpwnam	zgetpwnam
-#define	getpwuid	zgetpwuid
-#define	getgrnam	zgetgrnam
-#endif	/* USE_GETPWNAM */
+struct Zgroup {
+  const char *gr_name;
+  const char *gr_passwd;
+  gid_t gr_gid;
+  const char **gr_mem;
+};
 
 /* lib/allocate.c: */
 extern void      tfree __((const memtypes memtype));
