@@ -80,9 +80,16 @@ struct policy_user {
 #define P_A_TestDnsRBL		14
 #define P_A_MESSAGE		15
 #define P_A_LocalDomain		16
+#define P_A_InboundSizeLimit	17
+#define P_A_OutboundSizeLimit	18
+#define P_A_FullTrustNet        19
+#define P_A_TrustRecipients	20
+#define P_A_TrustWhosOn		21
+#define P_A_RcptDnsRBL		22
+#define P_A_TestRcptDnsRBL	23
 
 #define P_A_FirstAttr	        2
-#define P_A_LastAttr	        16
+#define P_A_LastAttr	        23
 /* Note: Attribute codes outside range 1..31 cause problems at policystate
          processing!  If you ever need modify these, fix the  policytest.c,
 	 and  policytest.h: struct policystate { char values[]; } array,
@@ -105,7 +112,7 @@ static char *_KK[] = {
 	"USER"
 };
 #define MAX_KK 5
-#define KK(x) ((((x)>0)&&((x)<=MAX_KK))?_KK[x]:"??")
+#define KK(x) ((((x)>0)&&((x)<=MAX_KK))?_KK[(x) & 0xFF]:"??")
 
 static char *_KA[] = {
 	"UNKNOWN",
@@ -125,7 +132,14 @@ static char *_KA[] = {
 	"test-dns-rbl",
 	"message",
 	"localdomain",
+	"maxinsize",
+	"maxoutsize",
+	"fulltrustnet",
+	"trustrecipients",
+	"trust-whoson",
+	"rcpt-dns-rbl",
+	"test-rcpt-dns-rbl"
 };
-#define KA(x) ((((x)>0)&&((x)<=P_A_LastAttr))?_KA[x]:"??")
+#define KA(x) ((((x)>0)&&((x)<=P_A_LastAttr))?_KA[(x) & 0xFF]:"??")
 
 #endif
