@@ -41,12 +41,14 @@ struct taddress {
 /* `convertmode' controls the behaviour of the message conversion:
      _CONVERT_NONE (0): send as is
      _CONVERT_QP   (1): Convert 8-bit chars to QUOTED-PRINTABLE
-     _CONVERT_8BIT (2): Convert QP-encoded chars to 8-bit
-     _CONVERT_UNKNOWN (3): Turn message to charset=UNKNOWN-8BIT, Q-P..
+     _CONVERT_MULTIPARTQP (2): Convert substructures to QP - when not it..
+     _CONVERT_8BIT (3): Convert QP-encoded chars to 8-bit
+     _CONVERT_UNKNOWN (4): Turn message to charset=UNKNOWN-8BIT, Q-P..
 */
 typedef enum {
   _CONVERT_NONE,
   _CONVERT_QP,
+  _CONVERT_MULTIPARTQP,
   _CONVERT_8BIT,
   _CONVERT_UNKNOWN
 } CONVERTMODE;
@@ -191,8 +193,8 @@ extern int append_header __((struct rcpt *rp, const char *fmt, ...));
 #else
 extern int append_header __(());
 #endif
-extern struct cte_data *parse_content_encoding __((char **cte_linep));
-extern struct ct_data  *parse_content_type __((char **ct_linep));
+extern struct cte_data *parse_content_encoding __((const char *cte_line));
+extern struct ct_data  *parse_content_type __((const char *ct_line));
 extern void output_content_type __((struct rcpt *rp, struct ct_data *ct, char **oldpos));
 extern int check_conv_prohibit __((struct rcpt *rp));
 extern int cte_check __((struct rcpt *rp));
