@@ -10,6 +10,8 @@
  *  Key is a NUL TERMINATED STRING (that is, string is: "XXXX\000")
  */
 
+/* This is incomplete ! */
+
 #include "hostenv.h"
 #include <pwd.h>
 #include <grp.h>
@@ -78,6 +80,8 @@ dbgetpwnam(name)
 	if (!dbpwd_init)
 	  init_dbpwd();
 
+	errno = 0;
+
 	if (!db_pwdnam)
 	  return getpwnam(name);
 
