@@ -71,8 +71,6 @@ union sockaddr_uni {
 #include "zmalloc.h"
 #include "libz.h"
 
-extern char  *getzenv     __((const char *));
-
 /*
 **  LOADIFADDRESSES -- load interface-specific addresses
 */
@@ -142,7 +140,7 @@ struct sockaddr ***sockaddrp;
 		return -5; /* UAARGH! */
 	      }
 
-	      if (sa->sa.sa_family == AF_INET) {
+	      if (sa->sa_family == AF_INET) {
 		struct sockaddr_in *si4 = (void*)malloc(sizeof(*si4));
 		if (si4 == NULL)
 		  break;
@@ -153,7 +151,7 @@ struct sockaddr ***sockaddrp;
 	      }
 
 #if defined(AF_INET6) && defined(INET6)
-	      if (sa->sa.sa_family == AF_INET6) {
+	      if (sa->sa_family == AF_INET6) {
 		struct sockaddr_in6 *si6 = (void*)malloc(sizeof(*si6));
 		if (si6 == NULL)
 		  break;
@@ -181,14 +179,9 @@ struct sockaddr ***sockaddrp;
 	  struct lifconf lifc;
 	  struct ifconf ifc;
 	  int ifbufsize = 4 * sizeof(struct lifreq) + 4;
-	  char *interfacebuf = (void*)malloc(ifbufsize);
+	  char *interfacebuf = NULL;
 	  int s;
 
-	  if (!interfacebuf) {
-	    free(sap);
-	    return -2;
-	  }
-
 	  s = socket(PF_INET6, SOCK_DGRAM, 0);
 	  if (s < 0)
 	    goto done_this_ipv6;
@@ -259,6 +252,8 @@ struct sockaddr ***sockaddrp;
 #ifdef SIOCGLIFFLAGS
 
 	    memset(&lifrf, 0, sizeof(struct lifreq));
+	    /* lifrf.lifr_index = 0;
+	       -- Solaris has _index, Linux has _ifindex */
 	    strncpy(lifrf.lifr_name, lifr->lifr_name, sizeof(lifrf.lifr_name));
 
 	    if (ioctl(s, SIOCGLIFFLAGS, (char *) &lifrf) < 0)
@@ -286,7 +281,7 @@ struct sockaddr ***sockaddrp;
 
 	    if (! sap) {
 	      close(s);
-	      free(interfacebuf);
+	      if (interfacebuf) free(interfacebuf);
 	      return -5; /* UAARGH! */
 	    }
 
@@ -312,7 +307,7 @@ struct sockaddr ***sockaddrp;
 done_this_ipv6:
 
 	  if (s >= 0) close(s);
-	  free(interfacebuf);
+	  if (interfacebuf) free(interfacebuf);
 
 	}
 #else /* SIOCGLIFCONF && IPv6 */
@@ -321,18 +316,12 @@ done_this_ipv6:
 	{
 	  struct ifconf ifc;
 	  int ifbufsize = 4 * sizeof(struct ifreq) + 4;
-	  char *interfacebuf = (void*)malloc(ifbufsize);
+	  char *interfacebuf = NULL;
 	  int s;
 
-	  if (!interfacebuf) {
-	    free(sap);
-	    return -2;
-	  }
-
 	  s = socket(PF_INET, SOCK_DGRAM, 0);
 
 	  if (s < 0) {
-	    free(interfacebuf);
 	    free(sap);
 	    return -1;
 	  }
@@ -401,6 +390,8 @@ done_this_ipv6:
 #ifdef SIOCGIFFLAGS
 
 	    memset(&ifrf, 0, sizeof(struct ifreq));
+	    /* ifrf.ifr_index = 0;
+	       -- Solaris has _index, Linux has _ifindex */
 	    strncpy(ifrf.ifr_name, ifr->ifr_name, sizeof(ifrf.ifr_name));
 
 	    if (ioctl(s, SIOCGIFFLAGS, (char *) &ifrf) < 0)
@@ -427,7 +418,7 @@ done_this_ipv6:
 	    }
 	    if (! sap) {
 	      close(s);
-	      free(interfacebuf);
+	      if (interfacebuf) free(interfacebuf);
 	      return -4; /* UAARGH! */
 	    }
 
@@ -441,8 +432,12 @@ done_this_ipv6:
 	    }
 	  }
 	  close(s);
-	  free(interfacebuf);
+	  if (interfacebuf) free(interfacebuf);
 	}
+#else
+
+#warning "NO SELF-ADDRESS EXTRACTION CODE AVAILABLE !?"
+
 #endif /* defined(SIOCGIFCONF) */
 #endif /* SIOCGLIFCONF / SIOCGIFCONF */
 
@@ -532,12 +527,10 @@ stashmyaddress(host)
 	  else
 	    addrsiz = -1;
 	}
-	if (myaddrs == NULL) {
-	  nmyaddrs = 0;
-	  myaddrs = (void*)malloc((nmyaddrs + naddrs +1) * sizeof(union sockaddr_uni*));
-	} else
-	  myaddrs = (void*)realloc((void*)myaddrs,
-				   (nmyaddrs + naddrs +1) * sizeof(union sockaddr_uni*));
+	/* malloc(size) == realloc(NULL, size) */
+	myaddrs = (void*)realloc((void*)myaddrs,
+				 (nmyaddrs + naddrs +1) *
+				 sizeof(union sockaddr_uni*));
 
 	if (!myaddrs) return; /* Uurgh.... */
 
@@ -574,7 +567,7 @@ void
 stashmyaddresses(host)
 const char *host;
 {
-	char *s1, *s2, *zenv;
+	const char *s1, *s2, *zenv;
 	union sockaddr_uni **sa;
 	int sacnt;
 
@@ -613,9 +606,9 @@ const char *host;
 	s1 = zenv;
 	while (s1 && *s1) {
 	  s2 = strchr(s1,',');
-	  if (s2) *s2 = 0;
+	  if (s2) *(char *)s2 = 0; /* Urgh... mumble mumble... */
 	  stashmyaddress(s1);
-	  if (s2) *s2 = ',';
+	  if (s2) *(char *)s2 = ',';
 	  if (s2)
 	    s1 = s2+1;
 	  else
@@ -671,10 +664,12 @@ int
 matchmyaddresses(ai)
 	struct addrinfo *ai;
 {
+	int i;
 	for ( ; ai ; ai = ai->ai_next ) {
-	  int i = matchmyaddress(ai->ai_addr);
-	  if (i != 0)
-	      return i;
+	  i = 0;
+	  if (ai->ai_addr)
+	    i = matchmyaddress(ai->ai_addr);
+	  if (i) return i;
 	}
 	return 0;
 }
