@@ -27,7 +27,7 @@
 #include	<stdio.h>
 #include	"hostenv.h"
 #include	"splay.h"
-#include	"malloc.h"
+#include	"zmalloc.h"
 
 #ifdef	MALLOC_TRACE
 #undef	sp_install
@@ -418,6 +418,7 @@ sp_init()
 
 	q = (struct sptree *) emalloc(sizeof(*q));
 
+	q->symbols = NULL;
 	q->eltscnt = 0;
 	q->lookups = 0;
 	q->lkpcmps = 0;
