@@ -1,4 +1,4 @@
-/* $Id: align.h,v 1.10 1991/04/12 01:03:41 moraes Exp $ */
+/* $Id: align.h,v 1.2 1999/12/29 19:37:35 mea Exp $ */
 #ifndef __ALIGN_H__
 #define __ALIGN_H__
 /* 
@@ -26,7 +26,7 @@
 # define NALIGN 8
 #endif
 
-#if defined(__alpha) || (defined(_MIPS_SZPTR) && _MIPS_SZPTR == 64)
+#if defined(__alpha) || defined(__alpha__) || (defined(_MIPS_SZPTR) && _MIPS_SZPTR == 64)
 /* 64 bit */
 # define ALIGN long foo
 # define NALIGN 8
