@@ -1,5 +1,5 @@
 /*  Author: Mark Moraes <moraes@csri.toronto.edu> */
-/* $Id: defs.h,v 1.24 1993/05/23 05:10:52 moraes Exp $ */
+/* $Id: defs.h,v 1.3 1999/12/30 03:26:07 mea Exp $ */
 #ifndef __DEFS_H__
 #define __DEFS_H__
 /* 
@@ -99,6 +99,7 @@ union word { /* basic unit of storage */
 	char **cpp;
 	int *ip;
 	int **ipp;
+	void (*allocator)(void);
 	ALIGN;			/* alignment stuff - wild fun */
 };
 
@@ -176,7 +177,9 @@ typedef union word Word;
  *  free block header uses extra words in the block itself
  */
 #ifdef DEBUG
-# define HEADERWORDS		2	/* Start boundary tag + real size in bytes */
+# define HEADERWORDS		3	/* Start boundary tag
+					   + real size in bytes 
+					   + allocator address */
 #else /* ! DEBUG */
 # define HEADERWORDS		1	/* Start boundary tag */
 #endif /* DEBUG */
@@ -310,6 +313,12 @@ typedef union word Word;
 # define COUNTSIZE(nw)
 #endif
 
+#ifdef __GNUC__
+# define CALLER_RETURN_ADDRESS ((void*)__builtin_return_address(0))
+#else
+# define CALLER_RETURN_ADDRESS ((void*)0)
+#endif
+
 #ifndef USESTDIO
   /* 
    * Much better not to use stdio - stdio calls malloc() and can
@@ -332,7 +341,8 @@ typedef union word Word;
 # define PRTRACE(x) \
   if (_malloc_tracing) { \
 	(void) x; \
-	(void) fputs(_malloc_statsbuf, _malloc_statsfile); \
+	if (_malloc_statsfile) \
+	  (void) fputs(_malloc_statsbuf, _malloc_statsfile); \
   } else \
 	_malloc_tracing += 0
 #else
