@@ -98,6 +98,8 @@ freeio(fioop, mark)
 	}
 }
 
+#define NO_DEQUOTE_AT_SIFTS
+
 /* #define DEQUOTE_STICKY */
 
 STATIC char *dequote __((const char *str, int len));
@@ -122,6 +124,10 @@ dequote (str, len)
 				AND they fit running in-place! */
 #endif
 
+#ifdef NO_DEQUOTE_AT_SIFTS
+	memcpy(s0, str, len);
+	s0[len] = 0;
+#else
 	do {
 
 	  sp = str;
@@ -156,7 +162,7 @@ dequote (str, len)
 	  len = strlen(str);
 
 	} while (ep > s0 && *s0 == *ep && (*s0 == '"' || *s0 == '\''));
-
+#endif
 /* fprintf(stderr,"\"%s\"\n",s0); */
 
 #ifdef DEQUOTE_STICKY
@@ -1187,22 +1193,31 @@ tscanstring(s)
 	int len;
 	token822 *t;
 
+	/* fprintf(stderr,"tscanstring('%s') ", s); */
 	t = HDR_SCANNER(s);
 	if (t != NULL && t->t_next == NULL && t->t_type == String) {
 	  /* we need to de-quote the quoted-string */
 	  char *bp;
 	  const char *buf;
 	  len = TOKENLEN(t);
-	  buf = bp = (char *)tmalloc(len+1);
+	  buf = bp = (char *)tmalloc(len+3);
+#ifdef NO_DEQUOTE_AT_SIFTS
+	  *bp++ = '"';
+#endif
 	  for (cp = t->t_pname; (cp - t->t_pname) < len ; ++cp) {
 	    if ((*cp == '\\') && ((cp - t->t_pname) < len-1))
 	      *bp++ = *++cp;
 	    else
 	      *bp++ = *cp;
 	  }
+#ifdef NO_DEQUOTE_AT_SIFTS
+	  *bp++ = '"';
+#endif
 	  *bp = '\0';
+	  /* fprintf(stderr,"dequote -> '%s'", buf); */
 	  t = HDR_SCANNER(buf);
 	}
+	/* fprintf(stderr,"\n"); */
 	return t;
 }
 
@@ -1408,9 +1423,14 @@ interpret(Vcode, Veocode, Ventry, caller, retcodep, cdp)
 			} else if (*arg1 != '\0' || quote) {
 				int slen = strlen(arg1);
 #if 0
-				d = newstring(dupnstr(arg1,slen),slen);
-#else
+				/* XXX: If we can be sure of input
+				   (arg1) being on stabile storage..
+				   Command-line interpreter (shell)
+				   is not stabile storage, but script
+				   interpreter is! */
 				d = conststring(arg1,slen);
+#else
+				d = newstring(dupnstr(arg1,slen),slen);
 #endif
 			} else
 				break;	/* it is a null string! */
@@ -1471,7 +1491,7 @@ fprintf(stderr,"%s:%d &command->buffer = %p\n",__FILE__,__LINE__,&command->buffe
 		case sArgVpush:
 			if (command->buffer == NULL)
 				break;
-			d = expand(command->buffer);
+			d = expand(command->buffer,1);
 			if (command->argv == NULL && STRING(d)) {
 				/* what kind of command is this? */
 				functype(d->string,
@@ -1611,7 +1631,7 @@ fprintf(stderr,"%s:%d &command->buffer = %p\n",__FILE__,__LINE__,&command->buffe
 			if (command->buffer == NULL)
 				variable = NULL;
 			else {
-				variable = expand(command->buffer);
+				variable = expand(command->buffer,2);
 				variable = ncons(variable);
 			}
 
@@ -1643,7 +1663,7 @@ fprintf(stderr,"%s:%d &command->buffer = %p\n",__FILE__,__LINE__,&command->buffe
 		case sVariableAppend:
 			if (command->buffer == NULL)
 				break;
-			d = expand(command->buffer);
+			d = expand(command->buffer,3);
 			if (variable == NULL)
 				variable = ncons(d);
 			else if (car(variable) == NULL)
@@ -2012,9 +2032,14 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 				if (*arg1) {
 				  slen = strlen(arg1);
 #if 0
-				  car(l) = newstring(dupnstr(arg1,slen),slen);
-#else
+				/* XXX: If we can be sure of input
+				   (arg1) being on stabile storage..
+				   Command-line interpreter (shell)
+				   is not stabile storage, but script
+				   interpreter is! */
 				  car(l) = conststring(arg1,slen);
+#else
+				  car(l) = newstring(dupnstr(arg1,slen),slen);
 #endif
 				} else
 				  car(l) = conststring(uBLANK,0);
@@ -2040,15 +2065,12 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 			  command->buffer = conststring(uBLANK,0);
 			if (nsift >= 0)
 			  v_accessed = sift[nsift].accessed;
-#if 0
-			assign(d, command->buffer,
-			       cmd==sAssign ? (struct osCmd *)NULL : command);
-#else
+
 			if (cmd == sAssign)
 			  assign(d, command->buffer, NULL);
 			else
 			  assign(d, command->buffer, command);
-#endif
+
 			/* do NOT reset *retcodep here */
 			command->bufferp = &command->buffer;
 			command->argv    = NULL;
@@ -2234,9 +2256,14 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 			{
 			  int slen = strlen(arg1);
 #if 0
-			  tmp = newstring(dupnstr(arg1,slen),slen); /* must allocate a fresh! */
+			  /* XXX: If we can be sure of input
+			     (arg1) being on stabile storage..
+			     Command-line interpreter (shell)
+			     is not stabile storage, but script
+			     interpreter is! */
+			  tmp = conststring(arg1,slen);
 #else
-			  tmp = conststring(arg1, slen);
+			  tmp = newstring(dupnstr(arg1,slen),slen);
 #endif
 			}
 			cdr(d) = caar(envarlist);
@@ -2413,11 +2440,17 @@ std_printf("set %x at %d\n", re, cdp->rearray_idx);
 			  if (arg1 != NULL) {
 			    int slen = strlen(arg1);
 #if 0
+			    /* XXX: If we can be sure of input
+			       (arg1) being on stabile storage..
+			       Command-line interpreter (shell)
+			       is not stabile storage, but script
+			       interpreter is! */
 			    tmp = conststring(arg1,slen);
 #else
 			    tmp = newstring(dupnstr(arg1,slen),slen);
 #endif
 			    tmp->flags |= QUOTEDSTRING;
+
 			    /* cdr(tmp) = command->buffer; */
 			    *command->bufferp = tmp;
 			    command->bufferp = &cdr(tmp);
