@@ -28,6 +28,8 @@ struct cmddef commands[] = {
 #include "sh-out.i"
 };
 
+extern int newcell_gc_interval, D_conscell;
+
 int ncommands = sizeof commands / sizeof commands[0];
 
 struct sptree *spt_funclist = NULL;
@@ -35,10 +37,10 @@ struct sptree *spt_builtins = NULL;
 struct sptree *spt_searchpath;
 
 int saweof;
-FILE *runiofp = stdout;
+FILE *runiofp = NULL;
 conscell *commandline = NULL;	/* argument to -c option */
 const char *progname;
-struct osCmd avcmd;
+struct osCmd avcmd = { 0, };
 
 char shfl[CHARSETSIZE/NBBY];
 
@@ -105,12 +107,15 @@ zshinit(argc, argv)
 	int c, errflag, io, uid, loadit;
 	register struct shCmd *shcmdp;
 	memtypes oval;
-	
+
+	runiofp = stdout;
+
 	oval = stickymem;
 	stickymem = MEM_PERM;
 
 	/* check for funny business */
-	if ((uid = geteuid()) != getuid()) {
+	uid = geteuid();
+	if (uid != getuid()) {
 		fprintf(stderr, "%s: ruid != euid\n", argv[0]);
 		exit(1);
 	}
@@ -127,7 +132,7 @@ zshinit(argc, argv)
 		spt_builtins = sp_init();
 
 	for (shcmdp = &builtins[0]; shcmdp->name != NULL; ++shcmdp)
-	  sp_install(symbol(shcmdp->name), (void *)shcmdp, 0, spt_builtins);
+	  sp_install(symbol(shcmdp->name), (void *)shcmdp, 0L, spt_builtins);
 
 	TOKEN_NARGS(sBufferSet) = 1;
 	TOKEN_NARGS(sBufferAppend) = 1;
@@ -168,8 +173,7 @@ zshinit(argc, argv)
 	loadit = errflag = 0;
 	optind = 1;	/* not to be influenced by previous getopt()'s */
 	while (1) {
-		c = getopt(argc, (char*const*)argv,
-			   "CILMOPRSYc:l:isaefhkntuvx");
+		c = getopt(argc, (char**)argv, "CGILMOPRSYc:l:isaefhkntuvx");
 		if (c == EOF)
 		  break;
 		switch (c) {
@@ -185,15 +189,18 @@ zshinit(argc, argv)
 			} else
 				setopt(c, 1);
 			break;
+		case 'G':	/* conscell GC debugging */
+			newcell_gc_interval = 1;
+			D_conscell = 1;
 		case 'R':	/* runtime I/O */
 		case 'I':	/* interpreter (runtime interpretation) */
 		case 'Y':	/* just open the runiofp stream */
 			setopt(c, 1);
 			if (runiofp == stdout) {
 				io = open("/dev/tty", O_WRONLY, 0);
-				dup2(io, 19);
+				dup2(io, 60);
 				close(io);
-				runiofp = fdopen(19 /* out of the way */, "w");
+				runiofp = fdopen(60 /* out of the way */, "w");
 			}
 			break;
 		/* these are the normal shell-external flags */
@@ -260,19 +267,30 @@ zshinit(argc, argv)
 		SIGNAL_HANDLE(SIGINT, trap_handler);
 	}
 
+	avcmd.argv = NULL;
+
+	staticprot(& avcmd.argv);  /* LispGC */
+	staticprot(& commandline); /* LispGC */
+	staticprot(& envarlist);   /* LispGC */
+
 	avcmd.argv = s_listify(argc-optind, &argv[optind]);
+
 	if (isset('s')) {
-		conscell *d, *tmp;
-		d = conststring(progname);
+		conscell *d = NULL;
+		GCVARS1;
+		GCPRO1(d);
+		d = conststring(progname,strlen(progname));
 		s_push(d, avcmd.argv);
+		UNGCPRO1;
 	}
 
 	v_envinit();
 	v_set(PS2, DEFAULT_PS2);
-	if (uid == 0)
+	if (uid == 0) {
 		v_set(PS1, DEFAULT_ROOT_PS1);
-	else
+	} else {
 		v_set(PS1, DEFAULT_PS1);
+	}
 	path_flush();
 	mail_flush();
 	mail_intvl();
@@ -294,7 +312,8 @@ zshfree()
 {
 	sp_scan(xundefun, (struct spblk *)NULL, spt_funclist);
 	sp_null(spt_funclist);
-	s_free_tree(envarlist);
+	/* s_free_tree(envarlist); */
+	envarlist = NULL;
 }
 
 /* return no. of characters left to read from *cpp */
