@@ -266,6 +266,9 @@ lilian(yyyy, mm, dd)
 {
 	int ly, nnn, lil;
 
+	if (mm < 1) mm = 1; /* For BAD input this may be zero.. */
+	if (dd < 1) dd = 1; /* For BAD input this may be zero.. */
+
 	/* Determine day in year (nnn) */
 	ly = yyyy%4 == 0 ? 1 : 0;
 	if (yyyy%100 == 0) ly = 0;
@@ -663,7 +666,7 @@ again:	/* \relax */
 		century = this_century;
 	if (year <= 0 && !have_year) /* default year (wraparound buggy...) */
 		year += localtmptr->tm_year%100;
-	if (month < 0) 	/* default month */
+	if (month <= 0) 	/* default month */
 		month = localtmptr->tm_mon + 1;
 	if (dayinmonth < 0)	/* default day in month */
 		dayinmonth = localtmptr->tm_mday;
