@@ -2,7 +2,7 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  *
- *	Modifications/maintance, Matti Aarnio, over years 1990-2000
+ *	Modifications/maintance, Matti Aarnio, over years 1990-2001
  *
  *	'longestmatch' driver kissg@sztaki.hu 970209
  */
@@ -81,6 +81,7 @@ struct db_info {
 #define	DB_MAPTOUPPER	0x02
 #define	DB_MODCHECK	0x04
 #define DB_NEG_CACHE	0x08
+#define DB_PERCENTSUBST	0x10
 
 
 struct db_kind {
@@ -192,6 +193,7 @@ iccachemarkup(spl)
 	return 0;
 }
 
+static void cache_gc_markup_iterator __(( void (*mrkupfunc)(conscell*) ));
 static void
 cache_gc_markup_iterator(mrkupfunc)
      void (*mrkupfunc)__((conscell*));
@@ -239,7 +241,7 @@ run_relation(argc, argv)
 	proto_config
 		= db_kinds[sizeof db_kinds/(sizeof (struct db_kind))-1].config;
 	while (1) {
-		c = getopt(argc, (char*const*)argv, "CbilmnNpud:f:s:L:t:Te:");
+		c = getopt(argc, (char*const*)argv, "CbilmnNpud:f:s:L:t:Te:%");
 		if (c == EOF)
 			break;
 		switch (c) {
@@ -305,6 +307,9 @@ run_relation(argc, argv)
 		case 'u':	/* map all keys to uppercase */
 			proto_config.flags |= DB_MAPTOUPPER;
 			break;
+		case '%':
+			proto_config.flags |= DB_PERCENTSUBST;
+			break;
 		case 'F':	/* find routine */
 		case 'S':	/* search routine */
 		default:
@@ -702,14 +707,15 @@ run_db(argc, argv)
  */
 
 conscell *
-db(dbname, key)
-	const char *dbname, *key;
+db(dbname, argv10)
+	const char *dbname, *argv10[];
 {
 	register int keylen;
 	conscell *l, *ll, *tmp;
 	struct spblk *spl;
 	struct db_info *dbip;
 	char *realkey;
+	const char *key = argv10[0];
 	search_info si;
 	struct cache *cache;
 	unsigned long khash;
@@ -758,10 +764,13 @@ db(dbname, key)
 	} else
 	  khash = crc32(key);
 
-	si.file = dbip->file;
-	si.key  = key;
+	si.file    = dbip->file;
+	si.key     = key;
 	si.subtype = dbip->subtype;
-	si.ttl  = dbip->ttl;
+	si.ttl     = dbip->ttl;
+	si.argv10  = argv10;
+	si.argv1   = NULL;
+
 	if ((dbip->flags & DB_MODCHECK)
 	    && dbip->modcheckp != NULL && (*dbip->modcheckp)(&si)) {
 		if (dbip->close != NULL) {
@@ -826,8 +835,8 @@ db(dbname, key)
 				if (D_db)
 					fprintf(stderr, "... found in cache\n");
 				/* return a scratch value */
-				tmp = s_copy_chain(cache->value);
-				return tmp;
+				ll = s_copy_chain(cache->value);
+				goto post_subst;
 			}
 			pcache = &cache->next;
 		}
@@ -897,6 +906,8 @@ db(dbname, key)
 				free(realkey);
 			}
 			UNGCPRO3;
+			if (si.argv1) free((void*)si.argv1);
+			si.argv1 = NULL;
 			return NULL;
 		}
 	}
@@ -955,6 +966,95 @@ db(dbname, key)
 		ll = l;
 
 	UNGCPRO3;
+ post_subst:
+
+	if (ll && dbip->flags & DB_PERCENTSUBST) {
+	  char *buf, *b;
+	  const char *p, *s;
+	  int i;
+	  int do_subst = 0;
+
+	  slen = 0;
+
+	  p = ll->cstring;
+
+	  /* TODO: %0 .. %9 substitutions ? */
+
+	  /* Estimate the required size, can safely overestimate! */
+
+	  for (;*p;++p) {
+	    if (*p == '%' && ('0' <= p[1] && p[1] <= '9')) {
+	      ++p;
+	      switch (*p) {
+	      case '0':
+		s = si.key;
+		break;
+	      case '1':
+		s = si.argv1;
+		if (!s) s = si.argv10[1];
+		break;
+	      default:
+		i = (*p - '0');
+		if (si.argv1) --i;
+		s = si.argv10[i];
+		break;
+	      }
+	      if (s)
+		slen += strlen(s);
+	      if (s)
+		do_subst = 1;
+	      continue;
+	    }
+	    ++slen;
+	  }
+
+	  /* We have some work to do, not all plain joy.. */
+
+	  if (do_subst) {
+
+	    l = tmp = NULL;
+	    GCPRO3(l, ll, tmp);
+
+	    b = buf = malloc(slen + 3); /* probably 1 extra is enough .. */
+
+	    for (p = ll->cstring; *p; ++p) {
+	      if (*p == '%' && ('0' <= p[1] && p[1] <= '9')) {
+		++p;
+		switch (*p) {
+		case '0':
+		  s = si.key;
+		  break;
+		case '1':
+		  s = si.argv1;
+		  if (!s) s = si.argv10[1];
+		  break;
+		default:
+		  i = (*p - '1');
+		  if (si.argv1) --i;
+		  s = si.argv10[i];
+		  break;
+		}
+		if (s)
+		  while (*s) *b++ = *s++;
+		continue;
+	      }
+	      *b++ = *p;
+	    }
+	    *b = 0;
+
+	    l = newstring(dupnstr(buf, b-buf), b-buf);
+	    free(buf);
+
+
+	    if (si.argv1) free((void*)si.argv1);
+	    si.argv1 = NULL;
+
+	    ll = l;
+
+	    UNGCPRO3;
+	  }
+
+	}
 	return ll;
 }
 
@@ -1102,14 +1202,14 @@ find_domain(lookupfn, sip)
 #endif
 	/* iterate over the subdomains of the key */
 	for (cp = buf; *cp;) {
-		while (*cp && *cp != '.')
-			++cp;
-		while (*cp == '.')
-			++cp;
+		while (*cp && *cp != '.') ++cp;
+		while (*cp == '.')        ++cp;
 		if (*(cp-1) == '.') {
 			sip->key = cp-1;
 			l = (*lookupfn)(sip);
-			if (l != NULL) {
+			if (l) {
+				if (sip->argv1) free((void*)sip->argv1);
+				sip->argv1 = dupnstr(buf, sip->key - buf);
 #ifdef PREDOT_TEST
 #ifndef HAVE_ALLOCA
 				free(buf);
@@ -1141,8 +1241,11 @@ find_domain(lookupfn, sip)
 	/* Still failed ?  Try to look for "." */
 	sip->key = ".";
 	l = (*lookupfn)(sip);
-	if (l != NULL)
+	if (l) {
+		if (sip->argv1) free((void*)sip->argv1);
+		sip->argv1 = dupnstr(realkey, keylen);
 		return l;
+	}
 
 	return NULL;
 }
@@ -1160,6 +1263,7 @@ find_nodot_domain(lookupfn, sip)
 	search_info *sip;
 {
 	register const char *cp;
+	const char *realkey = sip->key;
 	conscell *l = NULL;
 
 	/* iterate over the subdomains of the key */
@@ -1171,8 +1275,12 @@ find_nodot_domain(lookupfn, sip)
 		if (*(cp-1) == '.') {
 			sip->key = cp;
 			l = (*lookupfn)(sip);
-			if (l != NULL)
+			if (l) {
+				if (sip->argv1) free((void*)sip->argv1);
+				sip->argv1 = dupnstr(realkey,
+						     sip->key - realkey);
 				return l;
+			}
 		}
 	}
 #if 0
@@ -1238,14 +1346,17 @@ find_longest_match(lookupfn, sip)
 				((h_addr&h_mask) >>  8) & 255,
 				((h_addr&h_mask)      ) & 255,
 				prefix);
-			if ((l = (*lookupfn)(sip)) != NULL)
+			l = (*lookupfn)(sip);
+			if (l)
 				return l;
 		}
-	}
-	else {	/* domain name */
+
+	} else {  /* domain name */
+
 		/* check the key as given */
 		if ((l = (*lookupfn)(sip)) != NULL)
 			return l;
+
 		realkey = (char *) sip->key;
 		/* iterate over the superdomains of the key */
 		for (cp = realkey; *cp;) {
@@ -1255,14 +1366,23 @@ find_longest_match(lookupfn, sip)
 				++cp;
 			if (*(cp-1) == '.') {
 				sip->key = cp-1;
-				if ((l = (*lookupfn)(sip)) != NULL)
+				l = (*lookupfn)(sip);
+				if (l) {
+					if (sip->argv1) free((void*)sip->argv1);
+					sip->argv1 = dupnstr(realkey,
+							     sip->key-realkey);
 					return l;
+				}
 			}
 		}
 		/* Still failed ?  Try to look for "." */
 		sip->key = ".";
-		if ((l = (*lookupfn)(sip)) != NULL)
+		l = (*lookupfn)(sip);
+		if (l) {
+			if (sip->argv1) free((void*)sip->argv1);
+			sip->argv1 = dupnstr(realkey, strlen(realkey));
 			return l;
+		}
 	}
 	
 	return NULL;
