@@ -1,7 +1,7 @@
 /*
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
- *	Some functions Copyright 1991-2000 Matti Aarnio.
+ *	Some functions Copyright 1991-2001 Matti Aarnio.
  */
 
 /*
@@ -87,8 +87,6 @@ static int run_malcontents ARGCV;
 
 extern const char *traps[];
 extern int nobody;
-extern struct group  *getgrnam __((const char *));
-extern struct passwd *getpwnam __((const char *));
 extern time_t time __((time_t *));
 extern int routerdirloops;
 
@@ -120,7 +118,7 @@ struct shCmd fnctns[] = {
 {	"printaliases",	run_praliases,	NULL,	NULL,	0	},
 {	"listaddresses",run_listaddresses,NULL,	NULL,	SH_ARGV	},
 {	"zapDSNnotify", zap_DSN_notify,	NULL,	NULL,	SH_ARGV	},
-{	"postzapDSNnotify", zap_DSN_notify, NULL, NULL,	SH_ARGV	},
+{	"postzapDSNnotify", post_zap_DSN_notify, NULL, NULL, SH_ARGV },
 {	"listexpand",	NULL,	run_listexpand,	NULL,	SH_ARGV	},
 #if 0
 {	"newattribute",	NULL,	run_newattribute, NULL,	SH_ARGV	},
@@ -350,13 +348,22 @@ run_dblookup(avl, il)
 	conscell *avl, *il; /* Inputs gc protected */
 {
 	conscell *l;
+	const char *argv10[10];
+	int i;
+
+	memset(argv10, 0, sizeof(argv10));
 
 	il = cdar(avl);
-	if (il == NULL || !STRING(il) || cdr(il) != NULL) {
-		fprintf(stderr, "Usage: %s key\n", car(avl)->string);
+	if (il == NULL || !STRING(il)) {
+		fprintf(stderr, "Usage: %s key [up_to_9_substitution_elements]\n", car(avl)->string);
 		return NULL;
 	}
-	if ((l = db(car(avl)->string, il->string)) == NULL)
+	i = 0;
+	for (; il && i < 9 && STRING(il); il = cdr(il))
+	  argv10[i++] = il->string;
+
+	l = db(car(avl)->string, argv10);
+	if (l == NULL)
 		return NULL;
 	return l;
 }
@@ -436,14 +443,14 @@ run_grpmems(argc, argv)
 	int argc;
 	const char *argv[];
 {
-	char **cpp;
-	struct group *grp;
+	const char **cpp;
+	struct Zgroup *grp;
 
 	if (argc != 2) {
 		fprintf(stderr, "Usage: %s groupname\n", argv[0]);
 		return 1;
 	}
-	grp = getgrnam(argv[1]);
+	grp = zgetgrnam(argv[1]);
 	if (grp == NULL) {
 		fprintf(stderr, "%s: no group '%s'\n", argv[0], argv[1]);
 		return 1;
@@ -1005,22 +1012,22 @@ run_listexpand(avl, il)
 	conscell *plustail = NULL, *domain = NULL;
 	conscell *l, *lrc;
 	char *localpart = NULL, *origaddr = NULL, *attributenam = NULL;
-	int c, n, errflag, stuff;
+	int   c, n, errflag, stuff;
 	volatile int cnt;
 	const char *comment, *erroraddress;
 	char *s, *s_end;
-	int privilege = -1;
+	int   privilege = -1;
 	struct siobuf *osiop = NULL;
 	FILE *mfp = NULL, *fp;
-	char buf[4096];
-	char DSNbuf[4096+200];
-	int fd2;
+	char  buf[4096];
+	char  DSNbuf[4096+200];
+	int   fd2;
 	char *olderrors = errors_to;
 	char *notary = NULL;
-	int no_dsn = 0;
-	int okaddresses = 0;
-	int errcount = 0;
-	int linecnt = 0;
+	int   no_dsn = 0;
+	int   okaddresses = 0;
+	int   errcount = 0;
+	int   linecnt = 0;
 	GCVARS3;
 
 	il = cdar(avl); /* The CDR (next) of the arg list.. */
@@ -1142,6 +1149,9 @@ run_listexpand(avl, il)
 		 */
 		if (zlinebuf[n-1] == '\n')
 			--n;
+		if (zlinebuf[0] == '#')
+			continue;
+
 		stuff = 0;
 		s_end = & zlinebuf[n];
 		for (s = zlinebuf; s < s_end; ++s) {
@@ -1290,9 +1300,13 @@ run_listexpand(avl, il)
 		    fprintf(mfp, "Precedence: junk\n\n");
 		    /* Print the report: */
 		    fprintf(mfp,"NO valid recipient addresses!\n");
+		    fprintf(mfp,"Verify source file protection/ownership/access-path\n");
+		    fprintf(mfp,"Current effective UID = %d\n", geteuid());
 		  }
 		} else { /* mfp != NULL */
 		  fprintf(mfp,"\nNO valid recipient addresses!\n");
+		  fprintf(mfp,"Verify source file protection/ownership/access-path\n");
+		  fprintf(mfp,"Current effective UID = %d\n", geteuid());
 		}
 	}
 
@@ -1310,9 +1324,16 @@ run_listexpand(avl, il)
 		int rc, slen;
 		memtypes omem;
 		char *s2, *se;
+		struct addr *pp;
+
+		for (pp = ap->a_tokens; pp != NULL; pp = pp->p_next)
+			if (pp->p_type == anAddress)
+				break;
+		if (pp == NULL)
+			continue;
 
 		buf[0] = 0;
-		pureAddressBuf(buf,sizeof(buf),ap->a_tokens);
+		pureAddressBuf(buf,sizeof(buf),pp);
 
 		if (buf[0] == 0) continue; /* Burp ??? */
 
@@ -1602,6 +1623,9 @@ run_listaddresses(argc, argv)
 		 */
 		if (zlinebuf[n-1] == '\n')
 			--n;
+		if (zlinebuf[0] == '#')
+			continue;
+
 		stuff = 0;
 		s_end = &zlinebuf[n];
 		for (s = zlinebuf; s < s_end; ++s) {
@@ -1746,9 +1770,13 @@ run_listaddresses(argc, argv)
 		    fprintf(mfp, "Precedence: junk\n\n");
 		    /* Print the report: */
 		    fprintf(mfp,"NO valid recipient addresses!\n");
+		    fprintf(mfp,"Verify source file protection/ownership/access-path\n");
+		    fprintf(mfp,"Current effective UID = %d\n", geteuid());
 		  }
 		} else { /* mfp != NULL */
 		  fprintf(mfp,"\nNO valid recipient addresses!\n");
+		  fprintf(mfp,"Verify source file protection/ownership/access-path\n");
+		  fprintf(mfp,"Current effective UID = %d\n", geteuid());
 		}
 
 	}
@@ -1759,13 +1787,15 @@ run_listaddresses(argc, argv)
 	}
 
 	for (ap = aroot; ap != NULL; ap = ap->a_next) {
+
 		for (pp = ap->a_tokens; pp != NULL; pp = pp->p_next)
 			if (pp->p_type == anAddress)
 				break;
 		if (pp == NULL)
 			continue;
-		pureAddress(stdout, ap->a_tokens);
-		/* printAddress(stdout, ap->a_tokens, 0); */
+
+		pureAddress(stdout, pp);
+		/* printAddress(stdout, pp, 0); */
 		putc('\n', stdout);
 	}
 	/*
@@ -1783,18 +1813,32 @@ run_homedir(argc, argv)
 	int argc;
 	const char *argv[];
 {
-	struct passwd *pw;
+	struct Zpasswd *pw;
+	char *b;
+	int err;
 
 	if (argc != 2) {
 		fprintf(stderr, "Usage: %s name\n", argv[0]);
 		return 1;
 	}
-	pw = getpwnam(argv[1]);
+
+	pw = zgetpwnam(argv[1]);
+	err = errno;
 	if (pw == NULL) {
 		strlower((char*)argv[1]);
-		pw = getpwnam(argv[1]);
-		if (pw == NULL)
-			return 2;
+		pw = zgetpwnam(argv[1]);
+		err = errno;
+		if (pw == NULL) {
+		  if (err == 0)      return 2;
+		  ++deferit;
+
+		  b = malloc(strlen(argv[1])+10);
+		  sprintf(b, "HOME:%s", argv[1]);
+		  v_set(DEFER, b);
+		  free(b);
+
+		  return 3;
+		}
 	}
 	printf("%s\n", pw->pw_dir);
 	return 0;
@@ -1824,6 +1868,8 @@ run_filepriv(argc, argv)
 	const char *argv0 = argv[0];
 	int id;
 	struct stat stbuf;
+	long fmode = -1;
+	long dmode = -1;
 	const char *file, *cp;
 	char *dir;
 	int maxperm = 0666 ^ filepriv_mask_reg; /* XOR.. */
@@ -1843,9 +1889,10 @@ run_filepriv(argc, argv)
 		return 1;
 	}
 	file = argv[1];
-	if (argc == 3 && isascii(argv[2][0]) && isdigit(argv[2][0]))
+	if (argc == 3 && isascii(argv[2][0]) && isdigit(argv[2][0])) {
+		fmode = S_IFREG|0400;
 		id = atoi(argv[2]);
-	else {
+	} else {
 		fd = open(file, O_RDONLY, 0);
 		if (fd < 0) {
 			/* if we can't open it, don't trust it */
@@ -1860,16 +1907,8 @@ run_filepriv(argc, argv)
 			close(fd);
 			return 3;
 		}
+		fmode = stbuf.st_mode;
 		close(fd);
-		if (!S_ISREG(stbuf.st_mode)
-		    || ((stbuf.st_mode & 07777) & ~maxperm) != 0) {
-			/*
-			 * If it is a  special or directory or writable
-			 * by non-owner (modulo permission), don't trust it!
-			 */
-			printf("%d\n", nobody);
-			return 0;
-		}
 		id = stbuf.st_uid;
 	}
 	cp = strrchr(file, '/');
@@ -1886,7 +1925,7 @@ run_filepriv(argc, argv)
 	memcpy(dir, file, cp - file);
 	dir[cp - file] = '\0';
 
-	/* Use stat(2), we fold symlinks to their final files */
+	/* Use stat(2), we fold symlinks to their final dirs(whatever) */
 	if (stat(dir, &stbuf) < 0 || !S_ISDIR(stbuf.st_mode)) {
 		fprintf(stderr, "%s: not a directory: \"%s\"!\n",
 			argv0, dir);
@@ -1898,10 +1937,54 @@ run_filepriv(argc, argv)
 #ifndef	USE_ALLOCA
 	free(dir);
 #endif
-	if (((stbuf.st_mode & filepriv_mask_dir) && /*If world/group writable*/
-	     !(stbuf.st_mode & S_ISVTX))	/* and not sticky, OR */
-	    || (stbuf.st_uid != 0 && stbuf.st_uid != id))/*owned by root/user*/
-		id = nobody;				 /* Don't trust */
+
+	dmode = stbuf.st_mode;
+
+	if (!S_ISDIR(dmode)) {
+	  /* Directory is not directory ?? */
+	  id = nobody;
+	}
+
+	if (id != nobody) {
+	  if (!S_ISREG(fmode))
+	    /* File is not a regular file ?? */
+	    id = nobody;
+	}
+
+	/*
+	 * If it is a  special or directory or writable
+	 * by non-owner (modulo permission), don't trust it!
+	 * BUT ONLY if the directory where it is has X bits
+	 * for group or others!
+	 */
+
+	/* Group and World accessibility of the residence directory
+	   defines what bits to analyze.
+	*/
+
+	switch (dmode & 00011) {
+	case 0000:	/* No Group access, no World access */
+	  if ((fmode & 07700) & ~maxperm) /* Verify only User bits */
+	    id = nobody;
+	  break;
+	case 0010:	/* Group access, no World access */
+	  if ((fmode & 07770) & ~maxperm) /* Verify User and Group bits */
+	    id = nobody;
+	  break;
+	case 0001:	/* No Group access, but yes World ?! */
+	case 0011:	/* Group and World accesses */
+	  if ((fmode & 07777) & ~maxperm) /* Verify all bits */
+	    id = nobody;
+	  break;
+	}
+
+	if ((dmode & filepriv_mask_dir) && /*If world/group writable*/
+	    !(dmode & S_ISVTX))	/* and not sticky, OR */
+	  id = nobody;				/* Don't trust */
+	if (id != nobody &&
+	    stbuf.st_uid != 0 && stbuf.st_uid != id)/*dir owned by root/user*/
+	  id = nobody;				/* Don't trust */
+
 	printf("%d\n", id);
 	return 0;
 }
