@@ -3,7 +3,7 @@
  *	This will be free software, but only when it is finished.
  *
  *	New database mechanism added by Matti Aarnio <mea@nic.funet.fi>
- *	over years 1992-1997
+ *	over years 1992-2001
  */
 
 /* Information needed by the database search routines */
@@ -20,6 +20,8 @@ typedef struct _search_info {
 	const char	*key;
 	time_t		 ttl;
 	const char	*subtype;
+	const char	**argv10;
+	const char	*argv1;
 } search_info;
 
 struct file_map {
@@ -58,15 +60,15 @@ extern conscell	*search_yp	__((search_info	*sip));
 extern conscell	*search_selfmatch __((search_info *sip));
 extern conscell	*search_ldap    __((search_info *sip));
 /* closes	*/
-extern void	close_core	__((search_info	*sip));
-extern void	close_seq	__((search_info	*sip));
-extern void	close_ndbm	__((search_info	*sip));
-extern void	close_btree	__((search_info	*sip));
-extern void	close_bhash	__((search_info	*sip));
-extern void	close_dbm	__((search_info	*sip));
-extern void	close_gdbm	__((search_info	*sip));
-extern void	close_header	__((search_info	*sip));
-extern void	close_ldap	__((search_info	*sip));
+extern void	close_core	__((search_info	*sip, const char *cmt));
+extern void	close_seq	__((search_info	*sip, const char *cmt));
+extern void	close_ndbm	__((search_info	*sip, const char *cmt));
+extern void	close_btree	__((search_info	*sip, const char *cmt));
+extern void	close_bhash	__((search_info	*sip, const char *cmt));
+extern void	close_dbm	__((search_info	*sip, const char *cmt));
+extern void	close_gdbm	__((search_info	*sip, const char *cmt));
+extern void	close_header	__((search_info	*sip, const char *cmt));
+extern void	close_ldap	__((search_info	*sip, const char *cmt));
 /* adds	*/
 extern int	add_core	__((search_info *sip, const char *value));
 extern int	add_seq		__((search_info *sip, const char *value));
