@@ -28,7 +28,7 @@
 extern const char * const gs_name;
 extern const char * const monthname[];
 extern char	*prio_list[];
-#ifndef HAVE_STRERROR /* System has it, and propably has prototype too..
+#ifndef HAVE_STRERROR /* System has it, and probably has prototype too..
 			 IRIX 6.2 */
 extern char	*strerror __((const int errno));
 #endif
@@ -45,13 +45,13 @@ extern int	tolower __((const int));
 
 /* File: conf.c */
 extern int	 files_gid;
-extern char	*cf_suffix;
+extern const char *cf_suffix;
 extern const char *default_trusted[];
 extern char	*err_prio_list[];
-extern char	*files_group;
-extern char	*logdir;
+extern const char *files_group;
+extern const char *logdir;
 extern char	*prio_list[];
-extern char	*trusted_group;
+extern const char *trusted_group;
 extern int	 filepriv_mask_reg;
 extern int	 filepriv_mask_dir;
 extern int	 maxReceived;
@@ -72,7 +72,7 @@ extern long	dateParse __((struct tm *localtmptr, token822 *t));
 extern int	run_relation __((int argc, const char *argv[]));
 extern struct sptree *icdbspltree __((const char *name));
 extern int	run_db __((int argc, const char *argv[]));
-extern conscell	  *db    __((const char *dbname, const char *key));
+extern conscell	  *db     __((const char *dbname, const char *argv10[]));
 extern const char *dbfile __((const char *dbname));
 extern void	   dbfree __((void));
 extern const char *dbtype __((const char *dbname));
