@@ -1219,7 +1219,7 @@ sequencer(e, file)
 	GCVARS5;
 
 	if (schedulersubdirhash < 0) {
-	  char *s = getzenv("SCHEDULERDIRHASH");
+	  const char *s = getzenv("SCHEDULERDIRHASH");
 	  if (s && ((s[0] == '1' || s[0] == '2') && s[1] == 0))
 	    schedulersubdirhash = s[0] - '0';
 	  else
@@ -1927,14 +1927,14 @@ sequencer(e, file)
 			GCVARS6;
 
 			/* secondlevel is XORs */
-#if 0
-			printf("crossbar sender: ");
-			s_grind(sender, stdout);
-			putchar('\n');
-			printf("crossbar to: ");
-			s_grind(to, stdout);
-			putchar('\n');
-#endif
+			if (D_sequencer) {
+			  printf("crossbar sender: ");
+			  s_grind(sender, stdout);
+			  putchar('\n');
+			  printf("crossbar to: ");
+			  s_grind(to, stdout);
+			  putchar('\n');
+			}
 
 			if ((x = crossbar(sender, to)) == NULL)
 			  continue;
@@ -2363,7 +2363,11 @@ sequencer(e, file)
 	sprintf(qpath, "../%s/%s%s", QUEUEDIR, subdirhash, file);
 	fflush(ofp);
 #ifdef HAVE_FSYNC
-	fsync(FILENO(ofp));
+	while (fsync(FILENO(ofp)) < 0) {
+	  if (errno == EINTR || errno == EAGAIN)
+	    continue;
+	  break;
+	}
 #endif
 	ofperrors |= ferror(ofp);
 
@@ -2403,7 +2407,11 @@ sequencer(e, file)
 		fprintf(vfp, "router done processing; F='%s' TF='%s'\n", file, path);
 		fflush(vfp);
 #ifdef HAVE_FSYNC
-		fsync(FILENO(vfp));
+		while (fsync(FILENO(vfp)) < 0) {
+		  if (errno == EINTR || errno == EAGAIN)
+		    continue;
+		  break;
+		}
 #endif
 		fclose(vfp);
 	}
