@@ -9,7 +9,7 @@
 #include "prototypes.h"
 
 conscell **return_valuep = NULL;
-conscell *s_value        = NULL ;
+conscell *s_value        = NULL;
 
 int
 l_apply(fname, l)
@@ -19,6 +19,9 @@ l_apply(fname, l)
 	int retval;
 	conscell *retvp = NULL;
 	conscell **oretvp = return_valuep;
+	GCVARS2;
+
+	GCPRO2(l, retvp);
 
 	return_valuep = &retvp;
 
@@ -26,6 +29,7 @@ l_apply(fname, l)
 	s_value = retvp;
 
 	return_valuep = oretvp;
+	UNGCPRO2;
 	return retval;
 }
 
@@ -38,12 +42,16 @@ s_apply(argc, argv)
 	int retval;
 	conscell *retvp = NULL;
 	conscell **oretvp = return_valuep;
+	GCVARS1;
+
+	GCPRO1(retvp);
 
 	return_valuep = &retvp;
 
 	retval = apply(argc, argv);
 	s_value = retvp;
 	return_valuep = oretvp;
+	UNGCPRO1;
 	return retval;
 }
 
@@ -66,12 +74,13 @@ n_apply(cpp, argc, argv)
  * This is used to rewrite an address.
  */
 
-static int s_rewrite __((const char *func, struct token *t, const char *argx));
+static int s_rewrite __((const char *func, token822 *t,
+			 const char *sender, const char *argx));
 
 static int
-s_rewrite(func, t, argx)
-	const char *func, *argx;
-	struct token *t;
+s_rewrite(func, t, sender, argx)
+	const char *func, *sender, *argx;
+	token822 *t;
 {
 	register char *cp, *bp;
 	const char *av[4];
@@ -82,7 +91,7 @@ s_rewrite(func, t, argx)
 	if (t == NULL)
 		return 0;
 	cp = buf;
-	cp += printdToken(&buf, &bufspc, t, (struct token *)NULL, 0);
+	cp += printdToken(&buf, &bufspc, t, (token822 *)NULL, 0);
 	/* Was it a quote-containing string ?  If so, strip the quotes,
 	   and undo back-slash quoting */
 	if (t->t_next == NULL && t->t_type == String && buf[0] == '"') {
@@ -120,15 +129,15 @@ s_rewrite(func, t, argx)
  * ignored.
  */
 
-static struct addr *mergeAddress __((struct addr *pp, struct token *t));
+static struct addr *mergeAddress __((struct addr *pp, token822 *t));
 
 static struct addr *
 mergeAddress(pp, t)
 	struct addr *pp;
-	struct token *t;
+	token822 *t;
 {
 	struct addr *ppp, *npp, *fpp;
-	struct token *nt, *pt;
+	token822 *nt, *pt;
 
 	pt = NULL;
 	for (ppp = fpp = NULL; pp != NULL; pp = pp->p_next, ppp = npp) {
@@ -169,14 +178,15 @@ hdr_rewrite(name, h)
 {
 	register struct address *ap;
 	register struct addr *pp;
-	register struct token *t;
+	register token822 *t;
 	struct address *nap = NULL, *pap;
-	struct token *nt, *pt, *addrtokens;
+	token822 *nt, *pt, *addrtokens;
 	struct header *nh;
-	conscell *l;
 	const char *cp, *eocp;
 	char *s, buf[4096], *eobuf; 	/* XX */
 
+	eobuf = buf + sizeof(buf)-1;
+
 	if (D_hdr_rewrite) {
 		printf("---------------------------\n");
 		printf("Sending this through %s:\n", name);
@@ -209,7 +219,8 @@ hdr_rewrite(name, h)
 			for (cp = addrtokens->t_pname, s=buf; cp < eocp; ++cp) {
 				if (*cp == '\\' && cp < eocp - 1 && *(cp+1) == '"')
 					continue;
-				*s++ = *cp;
+				if (s < eobuf)
+				  *s++ = *cp;
 			}
 			*s = 0;
 			addrtokens->t_pname = strsave(buf);
@@ -230,34 +241,43 @@ hdr_rewrite(name, h)
 		 */
 		if (addrtokens == NULL)
 			s_value = NULL;
-		else if (s_rewrite(name, addrtokens, h->h_pname) != 0) {
+		else if (s_rewrite(name, addrtokens, NULL, h->h_pname) != 0) {
 			if (s_value != NULL) {
-				s_free_tree(s_value);
-				s_value = NULL;
+			  /* s_free_tree(s_value); */
+			  s_value = NULL;
 			}
 			if (deferit
-			    && s_rewrite(DEFERHDR, addrtokens, h->h_pname)
+			    && s_rewrite(DEFERHDR, addrtokens, NULL, h->h_pname)
 			    && s_value != NULL) {
-				s_free_tree(s_value);
-				s_value = NULL;
+			  /* s_free_tree(s_value); */
+			  s_value = NULL;
 			}
 		}
 		if (s_value != NULL
 		    && (LIST(s_value) || *(s_value->string) == '\0')) {
-			s_free_tree(s_value);
-			s_value = NULL;
+		  /* s_free_tree(s_value); */
+		  s_value = NULL;
 		}
 		if (s_value == NULL) {
 			/* copy this address unchanged */
 			nap->a_tokens = ap->a_tokens;
 		} else {
 			/* integrate result with original address form */
-			l = s_copy_tree(s_value);
-			s_free_tree(s_value);
-			s_value = NULL;
-			/* t = HDR_SCANNER(l->cstring); */
-			t = scan822(&l->cstring, strlen(l->cstring),
+
+			const char *cs = s_value->cstring;
+			char *s;
+			memtypes osticky = stickymem;
+			stickymem = MEM_TEMP;
+
+			/* This really does need long-term storage! */
+			s = tmalloc(strlen(cs)+1);
+			strcpy(s, cs);
+			/* t = HDR_SCANNER(cs); */
+			t = scan822((const char**)&s, strlen(s),
 				    '!', '%', 0, &ap->a_tokens->p_tokens);
+
+			stickymem = osticky;
+
 			/* X: check for errors! */
 			nap->a_tokens = mergeAddress(ap->a_tokens, t);
 		}
@@ -280,21 +300,24 @@ setenvinfo(e)
 	struct envelope *e;
 {
 	struct header *h;
-	conscell *pl, *plhead, *tmp;
+	conscell *pl, *plhead;
 	char buf[20];
+	GCVARS1;
+	int slen;
 
 	/* include header size ("headersize"), message size ("size"),
 	   message body size ("bodysize"), now ("now"), resent ("resent")
 	   trusted ("trusted"), message file name ("file"), message id
 	   ("message-id") */
 
-#define	CONSTSTR(s)	cdr(pl) = conststring(s); pl = cdr(pl)
-#define	NEWSTR(s)	cdr(pl) = newstring(s); pl = cdr(pl)
-#define	NEWSTRD(d)	sprintf(buf, "%ld", (long)(d)); \
-			cdr(pl) = newstring(strsave(buf)); \
-			pl = cdr(pl)
+#define	CONSTSTR(s)	slen = strlen(s); cdr(pl) = conststring(s, slen); pl = cdr(pl)
+#define	NEWSTR(s)	slen = strlen(s); cdr(pl) = newstring(s, slen);   pl = cdr(pl)
+#define	NEWDUPSTR(s)	slen = strlen(s); cdr(pl) = newstring(dupnstr(s, slen), slen);   pl = cdr(pl)
+#define	NEWSTRD(d)	sprintf(buf, "%ld", (long)(d)); NEWDUPSTR(buf)
+
+	pl = plhead = conststring("file", 4);
+	GCPRO1(plhead);
 
-	pl = plhead = conststring("file");
 	CONSTSTR(e->e_file);
 
 	if (e->e_messageid != NULL) {
@@ -345,6 +368,7 @@ setenvinfo(e)
 	cdr(pl) = NULL;
 	plhead = ncons(plhead);
 	v_setl("envelopeinfo", plhead);
+	UNGCPRO1;
 }
 
 static char gsbuf[30];
@@ -352,90 +376,108 @@ static char gsbuf[30];
  *  newattribute_2()
  */
 char *newattribute_2(onam,nam,val)
-const char *onam, *nam, *val;
+     const char *onam, *nam, *val;
 {
 	conscell *l, *lc, *tmp, **pl;
 	conscell	*l1;
+	GCVARS4;
+	int slen;
 
 	l1 = v_find(onam);
 	if (!l1)
 	  return NULL;
 	l = copycell(cdr(l1));
+	lc = tmp = l1 = NULL;
+	GCPRO4(l, lc, tmp, l1);
+
 	cdr(l) = NULL;
-	car(l) = s_copy_tree(car(l));
+	car(l) = s_copy_chain(car(l));
 	pl = &car(l);
 	l1 = *pl;
 	for (lc = l1; lc && cdr(lc); pl = &cddr(lc),lc = *pl) {
-	  if (!STRING(lc))
+	  if (!STRING(lc)) {
+	    UNGCPRO4;
 	    return NULL; /* ?? */
+	  }
 	  if (strcmp(nam,lc->string)==0) {
-	    if (!cdr(lc))
+	    if (!cdr(lc)) {
+	      UNGCPRO4;
 	      return NULL;
+	    }
 	    *pl = cddr(lc) /* Skip this in chain */;
 	  }
 	}
 
 	/* Prepend in reverse order */
-	tmp = newstring(strsave(val));
+	slen = strlen(val);
+	tmp = newstring(dupnstr(val, slen), slen);
 	cdr(tmp) = car(l);
 	car(l) = tmp;
-	tmp = newstring(strsave(nam));
+	slen = strlen(nam);
+	tmp = newstring(dupnstr(nam, slen), slen);
 	cdr(tmp) = car(l);
 	car(l) = tmp;
 
 	sprintf(gsbuf, gs_name, gensym++);
 	/* gX (name in gsbuf) will be freed by free_gensym() later */
 	v_setl(gsbuf, l);
+	UNGCPRO4;
 	return gsbuf;
 }
 
 /*
  * Build gensym
  */
-char *
-build_gensym(uid,type,DSNstr,errorsto,sender)
-int uid;
-const char *type, *DSNstr, *errorsto, *sender;
+static char  *build_gensym __((int, const char*, const char*, const char*, const char*, const char*, const char*));
+
+static char *
+build_gensym(uid, type, DSNstr, DSNret, DSNenv, errorsto, sender)
+     int uid;
+     const char *type, *DSNstr, *DSNret, *DSNenv, *errorsto, *sender;
 {
 	char buf[20];
-	conscell *l, *lc, *tmp;
+	conscell *l, *pl;
+	GCVARS1;
+	int slen;
 
 	/* assemble the default attribute list: (privilege <uid>) */
-	l = conststring("privilege");
+	l = conststring("privilege", 9);
+	GCPRO1(l);
 	sprintf(buf, "%d", uid);
-	cdr(l) = newstring(strsave(buf));
-	lc = cdr(l);
-	if (type != NULL) {
-		cdr(lc) = conststring("type");
-		lc = cdr(lc);
-		cdr(lc) = newstring(strsave(type));
-		lc = cdr(lc);
+	pl = l;
+	NEWDUPSTR(buf);
+	if (type) {
+		CONSTSTR("type");
+		CONSTSTR(type); /* Always a constant string */
 	}
-	if (DSNstr != NULL) {
-		cdr(lc) = conststring("DSN");
-		lc = cdr(lc);
-		cdr(lc) = newstring(strsave(DSNstr));
-		lc = cdr(lc);
+	if (DSNstr) {
+		CONSTSTR("DSN");
+		NEWDUPSTR(DSNstr);
+	}
+	if (DSNret) {
+		CONSTSTR("DSNr");
+		NEWDUPSTR(DSNret);
+	}
+	if (DSNenv) {
+		CONSTSTR("DSNe");
+		NEWDUPSTR(DSNenv);
 	}
 	/* See if some "errorsto" definition is available.. */
-	if (errorsto != NULL) {
-		cdr(lc) = conststring("ERR");
-		lc = cdr(lc);
-		cdr(lc) = newstring(strsave(errorsto));
-		lc = cdr(lc);
+	if (errorsto) {
+		CONSTSTR("ERR");
+		NEWDUPSTR(errorsto);
 	}
 	/* See if some "sender" definition is available.. */
-	if (sender != NULL) {
-		cdr(lc) = conststring("sender");
-		lc = cdr(lc);
-		cdr(lc) = newstring(strsave(sender));
-		lc = cdr(lc);
+	if (sender) {
+		CONSTSTR("sender");
+		NEWDUPSTR(sender);
 	}
-	cdr(lc) = NULL;
+	cdr(pl) = NULL; /* not needed in reality */
 	l = ncons(l);
 	sprintf(gsbuf, gs_name, gensym++);
 	/* gX (name in gsbuf) will be freed by free_gensym() later */
 	v_setl(gsbuf, l);
+	UNGCPRO1;
 	return gsbuf;
 }
 
@@ -448,26 +490,33 @@ const char *type, *DSNstr, *errorsto, *sender;
  */
 
 conscell *
-router(a, uid, type)
+router(a, uid, type, senderstr)
 	struct address *a;
 	int uid;
-	const char *type;
+	const char *type, *senderstr;
 {
-	register struct token *t, *tt;
+	register token822 *t, *tt;
 	int r;
-	struct token *last;
+	token822 *last;
 	struct addr *p;
-	conscell *l, *tmp;
+	conscell *l;
 	const char *gsym;
 	struct notary *DSN = NULL;
 	const char *DSNstr;
+	const char *DSNret;
+	const char *DSNenv;
+	GCVARS1;
 
 	if (a == NULL)
 		return NULL;
 	t = last = NULL;
 	DSN = a->a_dsn;
-	DSNstr = NULL;
-	if (DSN) DSNstr = DSN->dsn;
+	DSNstr = DSNret = DSNenv = NULL;
+	if (DSN) {
+	  DSNstr = DSN->dsn;
+	  DSNret = DSN->ret;
+	  DSNenv = DSN->envid;
+	}
 	for (p = a->a_tokens; p != NULL; p = p->p_next)
 		if (p->p_type == anAddress) {
 			/* link up all address tokens together */
@@ -491,48 +540,52 @@ router(a, uid, type)
 	if (t->t_pname[0] == '<' && TOKENLEN(t) == 1 && t->t_next == NULL)
 		abort();
 
-	gsym = build_gensym(uid,type,DSNstr,errors_to,NULL);
+	gsym = build_gensym(uid, type, DSNstr, DSNret, DSNenv, errors_to, senderstr);
 
 	deferit = 0;
 	v_set(DEFER, "");
-	r = s_rewrite(ROUTER, t, gsym);
+	r = s_rewrite(ROUTER, t, NULL, gsym);
 #if 0
 	if (deferit) {
-		s_free_tree(s_value);
-		s_value = NULL;
-		r = s_rewrite(DEFERENV, t, gsym);
+	  /* s_free_tree(s_value); */
+	  s_value = NULL;
+	  r = s_rewrite(DEFERENV, t, NULL, gsym);
 	}
 #endif
 	if (r != 0 || s_value == NULL || !LIST(s_value)) {
-		/* router returned something invalid */
-		s_free_tree(s_value);
-		s_value = NULL;
-		return NULL;
+	  /* router returned something invalid */
+	  /* s_free_tree(s_value); */
+	  s_value = NULL;
+	  return NULL;
 	}
 
 	/*
 	 * We expect router to either return
 	 * (local - user attributes) or (((local - user attributes)))
 	 */
+	l = NULL;
+	GCPRO1(l);
 	if (car(s_value) && LIST(car(s_value))) {
 		if (!LIST(caar(s_value)) || !STRING(caaar(s_value))) {
 			fprintf(stderr,
 				"%s: '%s' returned invalid 2-level list: ",
 				progname, ROUTER);
 			s_grind(s_value, stderr);
-			s_free_tree(s_value);
+			/* s_free_tree(s_value); */
 			s_value = NULL;
+			UNGCPRO1;
 			return NULL;
 		}
-		l = s_copy_tree(s_value);
+		l = s_copy_chain(s_value);
 	} else {
-		l = s_copy_tree(s_value);
+		l = s_copy_chain(s_value);
 		l = ncons(l);
 		l = ncons(l);
 	}
 
-	s_free_tree(s_value);
+	/* s_free_tree(s_value); */
 	s_value = NULL;
+	UNGCPRO1;
 
 	return l;
 }
@@ -554,16 +607,19 @@ conscell *
 crossbar(from, to)
 	conscell *from, *to;
 {
-	conscell *l, *tmp;
+	conscell *l = NULL;
+	GCVARS3;
+
+	GCPRO3(l, from, to);
+
+	l = copycell(from);
+	l = ncons(l);
 
-	l = ncons(from);
 	cdar(l) = to;
-	s_set_prev(l, car(l));
 
 	if (l_apply(CROSSBAR, l) != 0 || s_value == NULL) {
-		if (s_value != NULL)
-			s_free_tree(s_value);	/* superfluous? */
-		return NULL;
+	  s_value = NULL;
+	  return NULL;
 	}
 
 	/*
@@ -573,9 +629,9 @@ crossbar(from, to)
 	 * back from the crossbar function.
 	 */
 
-	l = s_copy_tree(s_value);
-	s_free_tree(s_value);
+	l = s_value;
 	s_value = NULL;
+	UNGCPRO3;
 
 	return l;
 }
