@@ -122,7 +122,7 @@ char * procselect = NULL;	/* Non-null defines  channel/host specifier
 char *  procselhost = NULL;	/* Just spliced out 'host'-part of the above */
 extern int forkrate_limit;	/* How many forks per second ? */
 int	mailqmode = 1;		/* ZMailer v1.0 mode on mailq */
-char *  mailqsock = NULL;
+const char *  mailqsock = NULL;
 
 static int vtxprep_skip      = 0;
 static int vtxprep_skip_any  = 0;
@@ -468,11 +468,11 @@ main(argc, argv)
 	const char *argv[];
 {
 	struct ctlfile *cfp;
-	char *config, *cp;
+	const char *config, *cp;
 	int i, daemonflg, c, errflg, version, fd;
 	long offout, offerr;
 
-	char *t, *syslogflg;
+	const char *t, *syslogflg;
 
 #ifdef GLIBC_MALLOC_DEBUG__ /* memory allocation debugging with GLIBC */
 	old_malloc_hook = __malloc_hook;
@@ -717,12 +717,12 @@ main(argc, argv)
 	  config = emalloc(3 + (u_int)(strlen(mailshare)
 				       + strlen(progname)
 				       + strlen(qcf_suffix)));
-	  sprintf(config, "%s/%s.%s", mailshare, progname, qcf_suffix);
+	  sprintf((char*)config, "%s/%s.%s", mailshare, progname, qcf_suffix);
 	}
 	cehead = readconfig(config);
 	if (cehead == NULL) {
 	  cp = emalloc(strlen(config)+50);
-	  sprintf(cp, "null control file, propably errors in it: %s", config);
+	  sprintf((char*)cp, "null control file, probably errors in it: %s", config);
 	  die(1, cp);
 	  /* NOTREACHED */
 	}
@@ -1033,13 +1033,10 @@ dq_insert(DQ, ino, file, delay)
 	  if (dq->wrkspace <= dq->wrkcount) {
 	    /* Increase the space */
 	    dq->wrkspace = dq->wrkspace ? dq->wrkspace << 1 : 8;
-	    if (dq->stats == NULL)
-	      dq->stats = (struct dirstatname **)emalloc(sizeof(void*) *
-							 dq->wrkspace);
-	    else
-	      dq->stats = (struct dirstatname**)erealloc(dq->stats,
-							 sizeof(void*) *
-							 dq->wrkspace);
+	    /* malloc(size) = realloc(NULL,size) */
+	    dq->stats = (struct dirstatname**)erealloc(dq->stats,
+						       sizeof(void*) *
+						       dq->wrkspace);
 	  }
 	  dq->stats[dq->wrkcount] = dsn;
 	  dq->wrkcount += 1;
@@ -1050,13 +1047,10 @@ dq_insert(DQ, ino, file, delay)
 	  if (dq->wrkspace2 <= dq->wrkcount2) {
 	    /* Increase the space */
 	    dq->wrkspace2 = dq->wrkspace2 ? dq->wrkspace2 << 1 : 8;
-	    if (dq->stats2 == NULL)
-	      dq->stats2 = (struct dirstatname **)emalloc(sizeof(void*) *
-							  dq->wrkspace2);
-	    else
-	      dq->stats2 = (struct dirstatname**)erealloc(dq->stats2,
-							  sizeof(void*) *
-							  dq->wrkspace2);
+	    /* malloc(size) = realloc(NULL,size) */
+	    dq->stats2 = (struct dirstatname**)erealloc(dq->stats2,
+							sizeof(void*) *
+							dq->wrkspace2);
 	  }
 	  dq->stats2[dq->wrkcount2] = dsn;
 	  dq->wrkcount2 += 1;
@@ -1129,6 +1123,11 @@ static int dirqueuescan(dir, dq, subdirs)
 	/* Some changes lately, open the dir and read it */
 
 	dirp = opendir(dir);
+	if (!dirp) {
+	  sfprintf(sfstderr,"A 'this can never fail' opendir('%s') failed!; errno=%d (%s)\n",dir,errno,strerror(errno));
+	  return 0;
+	}
+
 	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
 	  /* Scan filenames into memory */
 
@@ -1732,7 +1731,7 @@ slurp(fd, ino)
 		/* cfp->erroraddr = strsave(s+2); */
 	      } else if (*s == _CF_TURNME) {
 		/* Umm... it is a bit complex */
-		sfprintf(sfstdout,"A TURNME request for target '%s'\n",s+2);
+		/* sfprintf(sfstdout,"An ETRN request for target '%s'\n",s+2); */
 	      }
 	    }
 	  }
@@ -2089,7 +2088,7 @@ static struct ctlfile *vtxprep(cfp, file, rereading)
 	      cfp->vfpfn = strsave(cp);
 	      break;
 	    case _CF_TURNME:
-	      sfprintf(sfstdout,"TURNME: %s\n",cp);
+	      sfprintf(sfstdout,"%s ETRN: %s\n", timestring(), cp);
 	      strlower(cp);
 	      turnme(cp);
 	      is_turnme = 1;
