@@ -414,6 +414,7 @@ int init;
 		printf("000- Alias-recursion: %d\n", recursions);
 
 	      strcpy(pbuf+2,str+2);
+	      strlower(pbuf+2);
 	      pbuf[0] = strlen(str+2) + 3;
 	      pbuf[1] = P_K_TAG;
 	      result = resolveattributes(rel, recursions, state, pbuf, 0);
@@ -692,9 +693,11 @@ int whosonrc;
     }
     if (!openok) {
 	/* ERROR!  Could not open the database! */
-      if (debug)
+      if (debug) {
 	printf("000- ERROR!  Could not open the database file '%s'; errno=%d!\n",
 	       dbname, errno);
+	fflush(stdout);
+      }
       *relp = NULL;
 
 #ifndef HAVE_ALLOCA
@@ -914,6 +917,7 @@ PolicyTest what;
 Usockaddr *raddr;
 {
     char pbuf[64]; /* Not THAT much space needed.. */
+    int rc;
 
     struct sockaddr_in *si4;
 #if defined(AF_INET6) && defined(INET6)
@@ -967,7 +971,9 @@ Usockaddr *raddr;
     }
 
     state->request = 0;
-    return _addrtest_(rel, state, pbuf, 1);
+    rc = _addrtest_(rel, state, pbuf, 1);
+    if (debug) fflush(stdout);
+    return rc;
 }
 
 
@@ -1388,6 +1394,7 @@ const int len;
     /* state->request initialization !! */
     state->request = ( 1 << P_A_RELAYTARGET     |
 		       1 << P_A_ACCEPTbutFREEZE |
+		       1 << P_A_TestRcptDnsRBL  |
 		       1 << P_A_LocalDomain );
 
     /* Test first the full address */
@@ -1405,6 +1412,23 @@ const int len;
 	PICK_PA_MSG(P_A_ACCEPTbutFREEZE);
 	return  1;
       }
+      if (valueeq(state->values[P_A_TestRcptDnsRBL], "+")) {
+
+	type(NULL, 0, NULL, "test-rcpt-dns-rbl test; rblmsg='%s'",
+	     state->rblmsg ? state->rblmsg : "<none>");
+
+	if (state->rblmsg != NULL) {
+	  /* Now this is cute... the source address had RBL entry,
+	     and the recipient domain had a request to honour the
+	     RBL data. */
+	  if (state->message != NULL) free(state->message);
+	  state->message = strdup(state->rblmsg);
+	  if (debug)
+	    printf("000- ... TestRcptDnsRBL has a message: '%s'\n",
+		   state->rblmsg);
+	  return -1;
+	}
+      }
     }
 
     /* state->request initialization !! */
@@ -1648,7 +1672,7 @@ const int len;
       abort();			/* Code error! Bad policy ! */
       return 9999; /* To silence most compilers.. */
     }
-    fflush(stdout);
+    if (debug) fflush(stdout);
     return rc;
 }
 
