@@ -4,7 +4,7 @@
  */
 /*
  *    Several extensive changes by Matti Aarnio <mea@nic.funet.fi>
- *      Copyright 1991-1999.
+ *      Copyright 1991-2000.
  */
 
 #include "smtpserver.h"
@@ -82,15 +82,18 @@ FILE *fp;
     return buf;
 }
 
+#ifndef HAVE_PUTENV
 static const char *newenviron[] =
-{"SMTPSERVER=y", NULL};
+  { "SMTPSERVER=y", NULL };
+#endif
 
 static int callr __((SmtpState * SS));
 static int callr(SS)
 SmtpState *SS;
 {
     int sawend, rpid = 0, to[2], from[2];
-    char *bufp, *cp;
+    char *bufp;
+    char *cp;
 
     if (pipe(to) < 0 || pipe(from) < 0)
 	return -1;
@@ -105,7 +108,7 @@ SmtpState *SS;
 #endif
 
     if (routerprog == NULL) {
-	if ((cp = getzenv("MAILBIN")) == NULL) {
+	if ((cp = (char *)getzenv("MAILBIN")) == NULL) {
 	    zsyslog((LOG_ERR, "MAILBIN unspecified in zmailer.conf"));
 	    return -1;
 	}
@@ -270,11 +273,11 @@ const int holdlast, len;
 	if (!bufp) {
 
 
-	    /* Huh! Got an EOF, while propably didn't expect it ?
+	    /* Huh! Got an EOF, while probably didn't expect it ?
 	       Lets find out what the subprocess status was */
 	    bufp = emalloc(80 + strlen(args0) + strlen(function) +
 			   sizeof(ROUTER_SERVER));
-	    sprintf(bufp, "400 Huh! Router server EOFed (crashed?) on us! Help! Last cmd was: %s %s \"%*s\"", ROUTER_SERVER, function, len, args0);
+	    sprintf(bufp, "400 Huh! Router server EOFed (crashed?) on us! Help! Last cmd was: %s %s \"%.*s\"", ROUTER_SERVER, function, len, args0);
 	    break;
 	}
 
