@@ -3,7 +3,8 @@
  *	This will be free software, but only when it is finished.
  */
 
-#include "hostenv.h"
+#include "mailer.h"
+
 #ifdef linux
 #define __USE_BSD 1
 #endif
@@ -12,6 +13,7 @@
 #include <sysexits.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#include <errno.h>
 
 #include "zmsignal.h"
 #include "zmalloc.h"
@@ -726,10 +728,16 @@ int
 hold_home(user)
 const char *user;
 {
-	struct passwd *pw;
+	struct Zpasswd *pw;
 	struct stat st;
 
-	if ((pw = getpwnam(user)) == NULL) return 1;
+	pw = zgetpwnam(user);
+	if (!pw)
+	  pw = zgetpwnam(user);
+	if (!pw) {
+	  if (errno == 0)      return 1;
+	  return ranny(2) == 0;	/* 30% of the time */
+	}
 	if (pw->pw_dir == NULL || pw->pw_dir[0] == '\0') return 1;
 	if (stat(pw->pw_dir, &st) == 0 &&
 	    S_ISDIR(st.st_mode)) return 1;
