@@ -19,7 +19,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include "zmsignal.h"
-#include "malloc.h"
+#include "zmalloc.h"
 
 #if	defined(HAVE_RESOLVER)
 #include <netdb.h>
@@ -39,6 +39,8 @@
 #endif
 #endif	/* HAVE_RESOLVER */
 
+#include <arpa/inet.h>
+
 #include "ta.h"
 
 #define DPRINTF(x)
@@ -541,10 +543,10 @@ getanswer_r(answer, anslen, qname, qtype, result)
 		}
 		if (type != qtype) {
 #if 0
-			syslog(LOG_NOTICE|LOG_AUTH,
+			zsyslog((LOG_NOTICE|LOG_AUTH,
 	       "gethostby*.getanswer: asked for \"%s %s %s\", got type \"%s\"",
 			       qname, p_class(C_IN), p_type(qtype),
-			       p_type(type));
+			       p_type(type)));
 #endif
 			cp += n;
 			continue;		/* XXX - had_error++ ? */
@@ -553,8 +555,8 @@ getanswer_r(answer, anslen, qname, qtype, result)
 		case T_PTR:
 			if (strcasecmp(tname, bp) != 0) {
 #if 0
-				syslog(LOG_NOTICE|LOG_AUTH,
-				       AskedForGot, qname, bp);
+				zsyslog((LOG_NOTICE|LOG_AUTH,
+				         AskedForGot, qname, bp));
 #endif
 				cp += n;
 				continue;	/* XXX - had_error++ ? */
@@ -564,7 +566,7 @@ getanswer_r(answer, anslen, qname, qtype, result)
 				had_error++;
 				break;
 			}
-#if MULTI_PTRS_ARE_ALIASES
+#ifdef MULTI_PTRS_ARE_ALIASES
 			cp += n;
 			if (!haveanswer)
 				host.h_name = bp;
@@ -596,8 +598,8 @@ getanswer_r(answer, anslen, qname, qtype, result)
 		case T_AAAA:
 			if (strcasecmp(result->host.h_name, bp) != 0) {
 #if 0
-				syslog(LOG_NOTICE|LOG_AUTH,
-				       AskedForGot, result->host.h_name, bp);
+				zsyslog((LOG_NOTICE|LOG_AUTH,
+				         AskedForGot, result->host.h_name, bp));
 #endif
 				cp += n;
 				continue;	/* XXX - had_error++ ? */
@@ -627,9 +629,10 @@ getanswer_r(answer, anslen, qname, qtype, result)
 				continue;
 			}
 			if (hap >= &result->h_addr_ptrs[MAXADDRS-1]) {
-				if (!toobig++)
+				if (!toobig++) {
 					DPRINTF(("Too many addresses (%d)\n",
 						 MAXADDRS));
+				}
 				cp += n;
 				continue;
 			}
@@ -724,7 +727,7 @@ gethostbyname2_rz(name, af, result)
 	 * disallow names consisting only of digits/dots, unless
 	 * they end in a dot.
 	 */
-	if (isdigit(name[0]))
+	if (isdigit(name[0] & 0xFF))
 		for (cp = name;; ++cp) {
 			if (!*cp) {
 				if (*--cp == '.')
@@ -753,7 +756,7 @@ gethostbyname2_rz(name, af, result)
 				h_errno = NETDB_SUCCESS;
 				return (&result->host);
 			}
-			if (!isdigit(*cp) && *cp != '.') 
+			if (!isdigit(*cp & 0xFF) && *cp != '.') 
 				break;
 		}
 
@@ -865,9 +868,9 @@ gethostbyaddr_rz(addr, len, af, result)
 	    _res.options |= RES_DEFNAMES;
 	    if (!(rhp = gethostbyname(hname2))) {
 #if 0
-		syslog(LOG_NOTICE|LOG_AUTH,
-		       "gethostbyaddr: No A record for %s (verifying [%s])",
-		       hname2, inet_ntoa(*((struct in_addr *)addr)));
+		zsyslog((LOG_NOTICE|LOG_AUTH,
+		         "gethostbyaddr: No A record for %s (verifying [%s])",
+		         hname2, inet_ntoa(*((struct in_addr *)addr))));
 #endif
 		_res.options = old_options;
 		h_errno = HOST_NOT_FOUND;
@@ -879,9 +882,9 @@ gethostbyaddr_rz(addr, len, af, result)
 			break;
 	    if (!*haddr) {
 #if 0
-		syslog(LOG_NOTICE|LOG_AUTH,
-		       "gethostbyaddr: A record of %s != PTR record [%s]",
-		       hname2, inet_ntoa(*((struct in_addr *)addr)));
+		zsyslog(LOG_NOTICE|LOG_AUTH,
+		        "gethostbyaddr: A record of %s != PTR record [%s]",
+		        hname2, inet_ntoa(*((struct in_addr *)addr))));
 #endif
 		h_errno = HOST_NOT_FOUND;
 		return (NULL);
