@@ -4,6 +4,8 @@
  */
 
 /*
+ *	Copyright 1994-2000 by Matti Aarnio
+ *
  * To really understand how headers (and their converted versions)
  * are processed you do need to draw a diagram.
  * Basically:
@@ -64,6 +66,8 @@ extern char *getzenv();
 # endif
 #endif
 
+extern char * dupnstr __((const char *str, const int len));
+
 static int mime_received_convert __((struct rcpt *rp, char *convertstr));
 
 /* extern FILE *verboselog; */
@@ -74,55 +78,91 @@ static int mime_received_convert __((struct rcpt *rp, char *convertstr));
 /* strqcpy() -- store the string into buffer with quotes if it
    		is not entirely of alphanums+'-'+'_'.. */
 
-static int strqcpy __((char *buf, int buflen, char *str));
+static int strqcpy __((char **buf, int startpos, int *buflenp, const char *str));
 
 static int
-strqcpy(buf,buflen,str)
-	char *buf, *str;
-	int buflen;
+strqcpy(bufp, startpos, buflenp, str)
+	char **bufp;
+	const char *str;
+	int *buflenp;
 {
-	char *s = str;
-	char *p = buf;
-	int cnt = buflen;
+	const char *s = str;
+	char *buf = *bufp;
 	int needquotes = 0;
+	int buflen = *buflenp -2;
+	int pos = startpos;
 
 	/* Copy while scanning -- redo if need quotes.. */
-	while (*s) {
+	for (; *s; ++s) {
 	  char c = *s;
-	  if (!((c >= '0' && c <= '9') ||
-		(c >= 'A' && c <= 'Z') ||
-		(c >= 'a' && c <= 'z') ||
-		c == '-' || c == '_')) {
+
+	  if (pos >= buflen) {
+	    *buflenp += 32;
+	    buflen = *buflenp -2;
+	    buf = realloc(buf, *buflenp + 5);
+	  }
+
+	  if (!(('0' <= c && c <= '9') ||
+		('A' <= c && c <= 'Z') ||
+		('a' <= c && c <= 'z') ||
+		c == '-' || c == '_' || c == '.')) {
 	    needquotes = 1;
 	    break;
 	  }
-	  if (cnt) {
-	    *p++ = c;
-	    --cnt;
-	  }
-	  ++s;
+	  buf[pos] = c;
+	  ++pos;
 	}
-	if (cnt && !needquotes) {
-	  *p = 0;
-	  return (buflen - cnt);
+	if (!needquotes) { /* Space left, not need quotes */
+	  buf[pos] = 0;
+	  *bufp = buf;
+	  return (pos);
 	}
-	/* Ok, need quotes.. */
-	p = buf;
-	cnt = buflen -3;
-	s = str;
-	*p++ = '"';
-	while (*s) {
-	  if (*s == '"' || *s == '\\')
-	    *p++ = '\\', --cnt;
-	  if (cnt>0)
-	    *p++ = *s, --cnt;
-	  ++s;
-	}
-	if (cnt<0) cnt = 0;
 
-	*p++ = '"';
-	*p   = 0;
-	return (buflen - cnt);
+	/* Ok, need quotes.. */
+	pos = startpos;
+	s = str;
+
+	if (pos+5 >= buflen) {
+	  *buflenp = pos + 32;
+	  buflen = *buflenp -2;
+	  buf = realloc(buf, *buflenp);
+	}
+
+	buf[pos] = '"';
+	++pos;
+
+	for (; *s; ++s) {
+	  char c = *s;
+
+	  if (pos >= buflen) {
+	    *buflenp = pos + 32;
+	    buflen = *buflenp - 4;
+	    buf = realloc(buf, *buflenp);
+	  }
+
+	  if (c == '"' || c == '\\') {
+	    buf[pos] = '\\';
+	    ++pos;
+	  }
+
+	  buf[pos] = c;
+	  ++pos;
+
+	}
+
+	if (pos+3 >= buflen) {
+	  *buflenp += pos+3;
+	  buflen = *buflenp;
+	  buf = realloc(buf, *buflenp);
+	}
+
+	buf[pos] = '"';
+	++pos;
+	buf[pos] = 0;
+
+	*bufp = buf;
+
+	return (pos);
 }
 
 
@@ -263,14 +303,40 @@ output_content_type(rp,ct,old)
 			attr_2="ajdsh 8327ead"
 	 */
 
-	char buf[200];
-	char *lines[40]; /* XX: Hopefully enough.. */
+	char **lines;
+	int  *linelens;
 	int  linecnt = 0, i;
+	int  linespace = 0;
 	char **newmsgheaders;
-	char **h1, **o1, **o2, ***basep;
-	int  oldcnt, hdrlines, newlines;
+	char **h1, **o2, ***basep;
+	int  hdrlines;
 	char *bp;
 	char **unk = ct->unknown;
+	int bufsiz, len, totlen = 0;
+	char *buf;
+
+	bufsiz  = 18;
+	if (ct->basetype)
+	  bufsiz += strlen(ct->basetype);
+	if (ct->subtype)
+	  bufsiz += strlen(ct->subtype) + 4;
+	if (ct->charset)
+	  bufsiz += strlen(ct->charset) + 14;
+
+	if (ct->boundary) {
+	  int siz2 = strlen(ct->boundary) + 16 ;
+	  if (siz2 > bufsiz) bufsiz = siz2;
+	}
+	if (ct->name) {
+	  int siz2 = strlen(ct->name) + 10 ;
+	  if (siz2 > bufsiz) bufsiz = siz2;
+	}
+
+	buf = malloc(bufsiz);
+
+	linespace = 6;
+	lines    = malloc(sizeof(lines[0]   ) * (linespace+2));
+	linelens = malloc(sizeof(linelens[0]) * (linespace+2));
 
 	sprintf(buf,"Content-Type:\t%s",ct->basetype);
 	bp = buf + strlen(buf);
@@ -282,67 +348,96 @@ output_content_type(rp,ct,old)
 	if (ct->charset) {
 	  strcat(bp,"; charset=");
 	  bp = bp + strlen(bp);
-	  strqcpy(bp,sizeof(buf)-1-(bp-buf),ct->charset);
+	  strqcpy(&buf, bp - buf, &bufsiz, ct->charset);
 	  bp = bp + strlen(bp);
-	  if (ct->boundary != NULL ||
-	      ct->name != NULL ||
-	      ct->unknown != NULL) {
-	    *bp++ = ';';
-	    *bp = 0;
-	  }
-	}
+	  if (ct->boundary || ct->name || unk)
+	    strcat(bp, ";");
+	} else if (ct->boundary || ct->name || unk)
+	  strcat(bp, ";");
+
 /*if (verboselog) fprintf(verboselog,"CT_out: '%s'\n",buf);*/
-	lines[linecnt++] = strdup(buf);
-	*buf = 0; bp = buf;
-	if (ct->boundary) {
-	  strcat(buf,"\tboundary=");
-	  bp = bp + strlen(bp);
-	  strqcpy(bp,sizeof(buf)-1-(bp-buf),ct->boundary);
-	  bp = bp + strlen(bp);
-	  if (ct->unknown != NULL) {
-	    *bp++ = ';';
-	    *bp = 0;
-	  }
-	}
-	if (*buf != 0) {
-/*if (verboselog) fprintf(verboselog,"CT_out: '%s'\n",buf);*/
-	  lines[linecnt++] = strdup(buf);
-	}
+	len = strlen(buf);
+	linelens[linecnt] = len;
+	lines[linecnt++] = dupnstr(buf, len);
+	totlen += len+1;
 
 	*buf = 0; bp = buf;
+
+	if (ct->boundary) {
+	  strcat(bp,"\tboundary=");
+	  bp = bp + strlen(bp);
+	  strqcpy(&buf, bp - buf, &bufsiz, ct->boundary);
+	  bp = bp + strlen(bp);
+	  if (ct->name || unk)
+	    strcat(bp, ";");
+
+/*if (verboselog) fprintf(verboselog,"CT_out: '%s'\n",buf);*/
+	  len = strlen(buf);
+	  linelens[linecnt] = len;
+	  lines[linecnt++] = dupnstr(buf, len);
+	  totlen += len + 1;
+	  *buf = 0; bp = buf;
+	}
+
 	if (ct->name) {
-	  strcat(buf,"\tname=");
+	  strcat(bp,"\tname=");
 	  bp = bp + strlen(bp);
-	  strqcpy(bp,sizeof(buf)-1-(bp-buf),ct->name);
+	  strqcpy(&buf, bp - buf, &bufsiz, ct->name);
 	  bp = bp + strlen(bp);
-	  if (ct->boundary != NULL ||
-	      ct->unknown != NULL) {
-	    *bp++ = ';';
-	    *bp = 0;
-	  }
-	}
-	if (*buf != 0) {
+	  if (unk)
+	    strcat(bp, ";");
+
 /*if (verboselog) fprintf(verboselog,"CT_out: '%s'\n",buf);*/
-	  lines[linecnt++] = strdup(buf);
+	  len = strlen(buf);
+	  linelens[linecnt] = len;
+	  lines[linecnt++] = dupnstr(buf, len);
+	  totlen += len +1;
+	  *buf = 0; bp = buf;
 	}
-	*buf = 0; bp = buf;
+
 	while (unk && *unk) {
-	  if (*buf) {
-	    /* There is something already, and more wants to push in.. */
-	    strcat(bp,";");
+	  int ulen = strlen(*unk);
+	  int pos = bp - buf;
+	  if (bufsiz < (pos + 2 + ulen)) {
+	    bufsiz = pos + 2 + ulen + 8;
+	    buf = realloc(buf, bufsiz);
+	    bp = buf + pos;
+	  }
+	  if ((pos + 2 + ulen) >= 78 && *buf) {
+	    /* There is something already, and more wants to push in,
+	       but would overflow the 78 column marker! */
+	    memcpy(bp, ";", 2);
 /*if (verboselog) fprintf(verboselog,"CT_out: '%s'\n",buf);*/
-	    lines[linecnt++] = strdup(buf);
+	    len = strlen(buf);
+	    if (linecnt >= linespace) {
+	      linespace <<= 1;
+	      lines    = realloc(lines,
+				 sizeof(lines[0]   ) * (linespace+2));
+	      linelens = realloc(linelens,
+				 sizeof(linelens[0]) * (linespace+2));
+	    }
+	    linelens[linecnt] = len;
+	    lines   [linecnt] = dupnstr(buf, len);
+	    ++linecnt;
+
+	    totlen += len +1;
 	    *buf = 0;
 	    bp = buf;
+	    pos = 0;
 	  }
 	  *bp++ = '\t';
-	  strcpy(bp,*unk);
+	  memcpy(bp, *unk, ulen);
+	  bp += ulen;
+	  *bp = 0;
 	  ++unk;
 	}
 
 	if (*buf != 0) {
 /*if (verboselog) fprintf(verboselog,"CT_out: '%s'\n",buf);*/
-	  lines[linecnt++] = strdup(buf);
+	  len = strlen(buf);
+	  linelens[linecnt] = len;
+	  lines[linecnt++] = dupnstr(buf, len);
+	  totlen += len +1;
 	}
 
 	/* The lines are formed, now save them.. */
@@ -354,18 +449,10 @@ output_content_type(rp,ct,old)
 	h1 = *basep;
 	for (; *h1; ++h1, ++hdrlines) ;
 
-	oldcnt = 0;
-	o1 = NULL;
-	if (old != NULL) {
-	  o1 = old;
-	  o2 = o1+1;
-	  while (*o2 != NULL && (**o2 == ' ' || **o2 == '\t'))
-	    ++o2, ++oldcnt;
-	}
-	newlines = hdrlines + linecnt - oldcnt + 1;
-	newmsgheaders = (char**)malloc(sizeof(char*)*newlines);
+	newmsgheaders = (char**)malloc(sizeof(char*)*(hdrlines +2));
 	if (! newmsgheaders) {
 	  zmalloc_failure = 1;
+	  if (buf) free(buf);
 	  return;
 	}
 	o2 = newmsgheaders;
@@ -376,10 +463,22 @@ output_content_type(rp,ct,old)
 	if (h1 == old) {
 	  /* Found the old entry ?  Skip over it.. */
 	  ++h1;
-	  while (*h1 && (**h1 == ' ' || **h1 == '\t')) ++h1;
 	}
-	for (i = 0; i < linecnt; ++i)
-	  *o2++ = lines[i];
+	buf = realloc(buf, totlen + 4);
+	bp = buf;
+	for (i = 0; i < linecnt; ++i) {
+	  memcpy(bp, lines[i], linelens[i]);
+	  free(lines[i]);
+	  bp += linelens[i];
+	  if (i+1 < linecnt)
+	    *bp++ = '\n';
+	}
+	*bp = 0;
+	*o2++ = buf; /* DO NOT FREE buf HERE! */
+
+	free(lines);
+	free(linelens);
+
 	while (*h1)
 	  *o2++ = *h1++;
 	*o2 = NULL;
@@ -390,14 +489,263 @@ output_content_type(rp,ct,old)
 	*basep = newmsgheaders;
 }
 
-struct ct_data *
-parse_content_type(ct_linep)
-     char **ct_linep;	/* Could be multiline! */
+static const char * skip_822linearcomments __((const char *p));
+
+#if 0 /* This code not needed */
+static const char * skip_822dtext __((const char *p));
+static const char * skip_822dtext(p)
+     const char *p;
 {
-	char *s, *p, *pv, *ss;
+	/* This shall do skipping of RFC 822:
+	   3.1.4: Structured Field Bodies  defined ATOMs */
+
+	const char *s = p;
+
+#warning "skip_822dtext() code missing!"
+
+	return s;
+}
+
+static const char * skip_822domainliteral __((const char *p));
+static const char * skip_822domainliteral(p)
+     const char *p;
+{
+	/* This shall do skipping of RFC 822: DOMAIN LITERALs */
+
+	const char *s = p;
+
+#warning " skip_822dliteral() missing code!"
+
+	return s;
+}
+#endif
+
+static const char * skip_822qtext __((const char *p));
+static const char * skip_822qtext(p)
+     const char *p;
+{
+	/* This shall do skipping of RFC 822:  qtext  items */
+
+	const char *s = p;
+
+	while (*s && *s != '"' && *s != '\\') ++s;
+
+	return s;
+}
+
+static const char * skip_822qpair __((const char *p));
+static const char * skip_822qpair(p)
+     const char *p;
+{
+	/* This shall do skipping of RFC 822:  quoted-pair  items */
+
+	const char *s = p;
+
+	++s; /* We come in with '\\' at *s */
+	/* End-of-header ?? */
+	if (*s != 0)
+	  ++s;
+
+	return s;
+}
+
+static const char * skip_822quotedstring __((const char *p));
+static const char * skip_822quotedstring(p)
+     const char *p;
+{
+	/* This shall do skipping of RFC 822:  quoted-string  items */
+
+	const char *s = p;
+
+	/* At arrival:  *s == '"' */
+	++s;
+
+	while (*s && *s != '"') {
+	  if (*s == '\\')
+	    s = skip_822qpair(s);
+	  else
+	    s = skip_822qtext(s);
+	}
+	if (*s == '"')
+	  ++s;
+
+	return s;
+}
+
+static const char * skip_comment __((const char *p));
+static const char * skip_comment(p)
+     const char *p;
+{
+	/* This shall do skipping of RFC 822:
+	   3.1.4: Structured Field Bodies
+	   defined COMMENTS */
+
+	const char *s = p;
+
+	++s; /* We are called with *s == '(' */
+	while (*s && *s != ')') {
+
+	  if (*s == '\\') { /* Quoted-Pair */
+	    s = skip_822qpair(s);
+	    continue;
+	  }
+	  if (*s == '(') {
+	    s = skip_comment(s);
+	    continue;
+	  }
+	  /* Anything else, just advance... */
+	  ++s;
+	}
+	if (*s == ')')
+	  ++s;
+
+	return s;
+}
+
+static const char * skip_822linearcomments(p)
+     const char *p;
+{
+	/* This shall do skipping of RFC 822:
+	   3.1.4: Structured Field Bodies
+	   defined LINEAR-WHITESPACES and COMMENTS */
+
+	const char *s = p;
+
+	while (*s) {
+
+	  /* To be exact we should scan over only SPC, TAB, CR and LF
+	     chars, but lets skip all controls just in case.. */
+	  while (0 <= *s && *s <= ' ') ++s; /* ASCII assumption! */
+
+	  if (*s == '(') {
+	    s = skip_comment(s);
+	    continue;
+	  }
+
+	  if (*s != 0) /* Not line-end */
+	    break; /* Anything else is worth ending the scan ? */
+	}
+
+	return s;
+}
+
+static const char * _skip_822atom __((const char *p, int tspecial));
+static const char * _skip_822atom(p, tspecial)
+     const char *p;
+     int tspecial;
+{
+	/* This shall do skipping of RFC 822:
+	   3.1.4: Structured Field Bodies  defined ATOMs
+
+	   - Sequence of non-control/space characters which
+	     end up at control/space/ATOM-special
+	   - ATOM-special chars
+
+	*/
+
+	const char *s = p;
+	int isdelim = 0;
+
+	while (*s) {
+	  char c;
+
+	  c = *s;
+
+	  /* We should *not* be called with DEL/Control/SPC! */
+	  if (c == 127 || (0 <= c && c <= ' '))
+	    break; /* Delimiters (controls, SPACE) */
+
+	  if (c == '.' && !tspecial) /* RFC 822 at odds with RFC 2045 */
+	    isdelim = 1;
+
+	  switch (c) { /* Any of specials ? */
+	  case '/':
+	  case '?':
+	  case '=': /* TokenSpecial as defined at MIME / RFC 2045 et.al. */
+	    if (!tspecial) break;
+	  case '(':
+	  case ')':
+	  case '\\':
+	  case '<':
+	  case '>':
+	  case '@':
+	  case ',':
+	  case ';':
+	  case ':':
+	  case '"':
+	  case '[':
+	  case ']':
+	    isdelim = 1;
+	  default:
+	    break;
+	  }
+
+	  /* If we have been scanning over non-atom special texts,
+	     and now found an atom-special, we break out! */
+	  if (s > p && isdelim) break;
+
+	  /* Now advance the character-in-question */
+	  ++s;
+
+	  /* We didn't have any gathered string so far, but if it
+	     happens to be special, we count them as single chars! */
+	  if (c == ':' && *s == ':')
+	    ++s; /* Consider '::' as single atom -- DECNET token */
+
+	  if (isdelim) break;
+	}
+
+	/* Just in case we got here with bad input, always
+	   return at least one char long sequence. */
+	if (s == p) ++s;
+
+	return s;
+}
+
+static const char * skip_mimetoken __((const char *p));
+static const char * skip_mimetoken(p)
+     const char *p;
+{
+  return _skip_822atom(p, 1);
+}
+
+
+static char * foldmalloccopy __((const char *start, const char *end));
+static char * foldmalloccopy (start, end)
+     const char *start;
+     const char *end;
+{
+	int len = end - start;
+	int space = len + 1;
+	char *b;
+
+	b = malloc(space);
+	if (!b) return NULL; /* UARGH! */
+
+	if (*start == '"' && len > 2) {
+	  /* QUOTED-STRING, which we UNQUOTE here */
+	  /* Our caller allows only quoted-strings, not
+	     quoted-localparts! */
+	  memcpy(b, start+1, len -2);
+	  len -= 2;
+	} else
+	  memcpy(b, start, len);
+	b[len] = 0;
+	return b;
+}
+
+
+struct ct_data *
+parse_content_type(ct_line)
+     const char *ct_line;
+{
+	const char *s, *p;
 	struct ct_data *ct = (struct ct_data*)malloc(sizeof(struct ct_data));
 	int unknowncount = 0;
 
+static const char *debugptrs[20];
+
+
 	if (!ct) return NULL; /* Failed to parse it! */
 
 	ct->basetype = NULL;
@@ -407,151 +755,101 @@ parse_content_type(ct_linep)
 	ct->name     = NULL;
 	ct->unknown  = NULL;
 
-	s = *ct_linep;
+	s = ct_line;
 	s += 13;	/* "Content-Type:" */
 
-	while (*s == ' ' || *s == '\t') ++s;
-	p = s;
-	while (*s && *s != ' ' && *s != '/' && *s != '\t' && *s != ';')
-	  ++s;
-	ct->basetype = malloc((s - p)+2);
-	/* FIXME: malloc problem check ?? */
-	ss = ct->basetype;
-	/* Copy over the basetype */
-	while (p < s) *ss++ = *p++;
-	*ss = 0;
-	while (*s == ' ' || *s == '\t') ++s;
-	if (*s == '/') {	/* Subtype defined */
-	  ++s;
-	  while (*s == ' ' || *s == '\t') ++s;
-	  p = s;
-	  while (*s && *s != ' ' && *s != '/' && *s != '\t' && *s != ';')
-	    ++s;
-	  ct->subtype = malloc((s - p)+2);
-	  /* FIXME: malloc problem check ?? */
-	  ss = ct->subtype;
-	  /* Copy over the subtype */
-	  while (p < s) *ss++ = *p++;
-	  *ss = 0;
+	debugptrs[0] = s;
+
+	p = skip_822linearcomments(s);
+	debugptrs[1] = p;
+	s = skip_mimetoken(p);
+	debugptrs[2] = s;
+
+	ct->basetype = foldmalloccopy(p, s);
+	debugptrs[3] = ct->basetype;
+
+	p = skip_822linearcomments(s);
+	debugptrs[4] = p;
+
+	if (*p == '/') {	/* Subtype defined */
+	  ++p;
+	  p = skip_822linearcomments(p);
+	  debugptrs[5] = p;
+	  s = skip_mimetoken(p);
+	  debugptrs[6] = p;
+
+	  ct->subtype = foldmalloccopy(p, s);
 	}
 
-	while (1) {
+	while (*s) {
 	  /* Check for possible parameters on the first and/or continuation
 	     line(s) of the header line... */
-	  char paramname[40];
-	  char *parval, c;
+	  char *paramname;
+	  char *parval;
 
-	  if (*s == 0) {
-	  /* Check if we have a continuation line */
-	    if ((s = ct_linep[1])) {
-	      if (*s != ' ' && *s != '\t') /* No continuation */
-		return ct;
-	      while (*s == ' ' || *s == '\t') ++s;
-	      if (*s == 0)
-		return ct;	/* No continuation,
-				   just a blank line w/ LWSP on it */
-	      ++ct_linep;	/* Advance.. */
-	    } else
-	      return ct;	/* Last of the header lines */
+	  debugptrs[7] = s;
+
+	  s = skip_822linearcomments(s);
+	  debugptrs[8] = s;
+
+	  if (!*s) break;
+	  if (*s == ';') ++s;
+	  else {
+	    /* Not found semicolon at expected phase ?? Whoo..
+	       Now shall we scan towards the end, and HOPE for the best,
+	       or what shall we do ? */
 	  }
-	  while (*s == ';' || *s == ' ' || *s == '\t') ++s;
-	  if (*s == 0) continue; /* last token on this line ? */
+	  debugptrs[9] = s;
+	  p = skip_822linearcomments(s);
+	  debugptrs[10] = p;
+	  if (!p || !*p) break;
+	  s = skip_mimetoken(p);
+	  debugptrs[11] = s;
+	  if (p == s && *s == 0) break; /* Nothing anymore */
 
-	  p = s;
-	  c = *s;
-	  while (c && (('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z') ||
-		       ('0' <= c && c <= '9') || c == '-' || c == '_'))
-	    c = *++s; /* Scan over a parameter name.. */
+	  paramname = foldmalloccopy(p, s);
+	  debugptrs[12] = paramname;
 
-	  /* Ok, `p' points to a parameter name string */
-	  if (s < (p+sizeof(paramname))) {
-	    strncpy(paramname,p,s-p);
-	    paramname[s-p] = 0;
-	  } else {
-	    strncpy(paramname,p,sizeof(paramname)-1);
-	    paramname[sizeof(paramname)-1] = 0;
-	  }
 	  /* Picked up a param name, now scan the value */
-	  pv = s;
+
+	  /* Have seen cases where there was:
+		charset = "foo-bar"
+	     That is, it had whitespaces around the "=" sign. */
+
+	  s = skip_822linearcomments(s);
+	  debugptrs[13] = s;
+
 	  if (*s == '=') {	    /* What if no `=' ?? */
-	    ++pv;
 	    ++s;
-	    if (*s == '"') {
-	      /* Scan a quoted string, stop at trailing '"' */
-	      int quoted = 0; /* Quoted with '\' */
-	      ++s;
-	      while (*s) {
-		if (!quoted && *s == '"') break;
-		if (*s == '\\') /* This quote skips over the next char */
-		  quoted = 2;
-		if (quoted) --quoted;
-		++s;
-	      }
-	      if (*s) ++s; /* Skip the trailing '"' */
-	    } else {
-	      /* Scan an alphanumeric string -- stop at ';', and LWSP */
-	      while (*s && *s != ';' && *s != ' ' && *s != '\t')
-		++s;
-	    }
+	  }
+	  debugptrs[14] = s;
+	  p = skip_822linearcomments(s);
+	  debugptrs[15] = p;
+
+	  if (*p == '"') {
+	    s = skip_822quotedstring(p);
 	  } else {
-	    /* XX: what if no `=' after the parameter name ? */
-	    /* ... it never happens, and then somebody screw things
-	       up in 9th of June, 1996, and nic.funet.fi experienced
-	       smtp crash.. */
-	    /* pv is at the start of the stuff as is.. */
-	    ++s;
-	    if (*s == '"') {
-	      /* Scan a quoted string, stop at trailing '"' */
-	      int quoted = 0; /* Quoted with '\' */
-	      ++s;
-	      while (*s) {
-		if (!quoted && *s == '"') break;
-		if (*s == '\\') /* This quote skips over the next char */
-		  quoted = 2;
-		if (quoted) --quoted;
-		++s;
-	      }
-	      if (*s) ++s; /* Skip the trailing '"' */
-	    } else {
-	      /* Scan an alphanumeric string -- stop at ';', and LWSP */
-	      while (*s && *s != ';' && *s != ' ' && *s != '\t')
-		++s;
-	    }
+	    s = skip_mimetoken(p);
 	  }
+	  debugptrs[16] = s;
 
-	  parval = malloc((s - pv) + 2);
-	  /* FIXME: malloc problem check ?? */
-	  ss = parval;
-	  if (*pv == '"') { /* Copy the quoted string to parval */
-	    int quoted = 0;
-	    ++pv;
-	    while (*pv && pv < s) {
-	      if (!quoted && *pv == '"') break;
-	      if (*pv == '\\') quoted = 2;
-	      if (quoted)    --quoted;
-	      if (!quoted)
-		*ss++ = *pv;
-	      ++pv;
-	    }
-	  } else { /* Copy the unquoted string to parval */
-	    while (pv < s)
-	      *ss++ = *pv++;
-	  }
-	  *ss = 0; /* Terminate the string */
+	  parval = foldmalloccopy(p, s);
+	  debugptrs[17] = parval;
 
-	  if (cistrcmp("charset",paramname)==0) {
+	  if (CISTREQ("charset",paramname)) {
 	    /* Parameter:  charset="..." */
 	    ct->charset = parval;
-	  } else if (cistrcmp("boundary",paramname)==0) {
+	  } else if (CISTREQ("boundary",paramname)) {
 	    /* Parameter:  boundary="...." */
 	    ct->boundary = parval;
-	  } else if (cistrcmp("name",paramname)==0) {
+	  } else if (CISTREQ("name",paramname)) {
 	    /* Parameter:  name="...." */
 	    ct->name     = parval;
 	  } else {
 	    /* Unknown parameter.. */
 	    int unklen = strlen(parval)+5+strlen(paramname);
 	    int unkpos;
+	    char *unk;
 	    if (!ct->unknown) {
 	      ct->unknown = (char**)malloc(sizeof(char*)*2);
 	/* FIXME: malloc problem check ?? */
@@ -561,46 +859,47 @@ parse_content_type(ct_linep)
 					    sizeof(char*)*(unknowncount+2));
 	/* FIXME: malloc problem check ?? */
 	    }
-	    ct->unknown[unknowncount] = malloc(unklen);
+	    unk = malloc(unklen);
 	/* FIXME: malloc problem check ?? */
-	    sprintf(ct->unknown[unknowncount],"%s=",paramname);
-	    unkpos = strlen(ct->unknown[unknowncount]);
-	    strqcpy(ct->unknown[unknowncount]+unkpos,unklen-unkpos,parval);
-	    ct->unknown[++unknowncount] = NULL;
+	    sprintf(unk, "%s=", paramname);
+	    unkpos = strlen(unk);
+	    strqcpy(&unk, unkpos, &unklen, parval);
+	    ct->unknown[unknowncount] = unk;
+	    ct->unknown[unknowncount] = NULL;
+	    ++unknowncount;
+	    free (parval);
 	  }
+	  if (paramname)
+	    free(paramname);
 	}
-	/*NOTREACHABLE*/
+	return ct;
 }
 
 struct cte_data *
-parse_content_encoding(cte_linep)
-     char **cte_linep;	/* Propably is not a multiline entry.. */
+parse_content_encoding(cte_line)
+     const char *cte_line;
 {
-	char *line, *s;
+	const char *s;
 	struct cte_data *cte = malloc(sizeof(struct cte_data));
 
 	if (!cte) return NULL;
 
-	s = (*cte_linep) + 26;
+	s = cte_line + 26;
 	/* Skip over the 'Content-Transfer-Encoding:' */
-	while (*s != 0 && (*s == ' ' || *s == '\t')) ++s;
-	line = s;
+	s = skip_822linearcomments(s);
 	if (*s == '"') {
-	  char *p;
-	  cte->encoder = p = malloc(strlen(s));
-	/* FIXME: malloc problem check ?? */
-	  while (*s && *s != '"')
-	    *p++ = *s++;
-	  *p = 0;
+	  const char *p = skip_822quotedstring(s);
+	  cte->encoder = foldmalloccopy(s, p);
+	  /* FIXME: malloc problem check ?? */
+	  s = p;
 	} else {
-	  char *p;
-	  cte->encoder = p = malloc(strlen(s));
-	/* FIXME: malloc problem check ?? */
-	  while (*s && *s != ' ' && *s != '\t')
-	    *p++ = *s++;
-	  *p = 0;
+	  const char *p = skip_mimetoken(s);
+	  cte->encoder = foldmalloccopy(s, p);
+	  /* FIXME: malloc problem check ?? */
+	  s = p;
 	}
-	while (*s == ' ' || *s == '\t') ++s;
+
+	/* while (*s == ' ' || *s == '\t') ++s; */
 	/* XX: if (*s) -- errornoeus data */
 
 	return cte;
@@ -620,11 +919,12 @@ check_conv_prohibit(rp)
 	if (!hdrs) return 0;
 
 	while (*hdrs) {
-	  if (cistrncmp(*hdrs,"Content-conversion:", 19)==0) {
-	    char *s = *hdrs + 19;
-	    while (*s == ' ' || *s == '\t') ++s;
-	    if (cistrncmp(s,"prohibited",10)==0) return -1;
-	    if (cistrncmp(s,"forced-qp",9)==0) return 7;
+	  if (CISTREQN(*hdrs,"Content-conversion:", 19)) {
+	    const char *s = *hdrs + 19;
+	    const char *p = skip_822linearcomments(s);
+	    if (*p == '"') ++p;
+	    if (CISTREQN(p,"prohibited",10)) return -2;
+	    if (CISTREQN(p,"forced-qp",9)) return 7;
 	    /* Prohibits (?) the content conversion.. */
 	  }
 	  ++hdrs;
@@ -651,15 +951,16 @@ cte_check(rp)
 	if (!hdrs) return 0;
 
 	while (*hdrs && (!mime || !cte)) {
-	  char *buf = *hdrs;
-	  if (!cte && cistrncmp(buf,cCTE,26)==0) {
+	  const char *buf = *hdrs;
+	  if (!cte && CISTREQN(buf,cCTE,26)) {
 	    buf += 26;
-	    while (*buf == ' ' || *buf == '\t') ++buf;
+	    buf = skip_822linearcomments(buf);
+	    if (*buf == '"') ++buf;
 	    if (*buf == '8' /* 8BIT */) cte = 8;
 	    else if (*buf == '7' /* 7BIT */) cte = 7;
 	    else if (*buf == 'Q' || *buf == 'q') cte = 9; /*QUOTED-PRINTABLE*/
 	    else cte = 1; /* Just something.. BASE64 most likely .. */
-	  } else if (!mime && cistrncmp(buf,"MIME-Version:",13)==0) {
+	  } else if (!mime && CISTREQN(buf,"MIME-Version:",13)) {
 	    mime = 1;
 	  }
 	  ++hdrs;
@@ -677,12 +978,12 @@ has_header(rp,keystr)
 	char **hdrs = *(rp->newmsgheader);
 	int keylen = strlen(keystr);
 
-	if (*(rp->newmsgheadercvt) != NULL)
+	if (*(rp->newmsgheadercvt))
 	  hdrs = *(rp->newmsgheadercvt);
 
 	if (hdrs)
 	  while (*hdrs) {
-	    if (cistrncmp(*hdrs,keystr,keylen)==0) return hdrs;
+	    if (CISTREQN(*hdrs,keystr,keylen)) return hdrs;
 	    ++hdrs;
 	  }
 	return NULL;
@@ -697,10 +998,6 @@ delete_header(rp,hdrp)	/* Delete the header, and its possible
 	char **h1 = hdrp;
 	char **h2 = hdrp+1;
 	ctlfree(rp->desc,*hdrp);
-	while (*h2 && (**h2 == ' ' || **h2 == '\t')) {
-	  ctlfree(rp->desc,*h2);
-	  ++h2;
-	}
 	while (*h2)
 	  *h1++ = *h2++;
 	/* And one more time.. To copy the terminating NULL ptr. */
@@ -721,16 +1018,16 @@ downgrade_charset(rp, verboselog)
 	CTE  = has_header(rp,cCTE);
 	if (CT == NULL || CTE == NULL) return 0; /* ??? */
 
-	ct = parse_content_type(CT);
+	ct = parse_content_type(*CT);
 
 	if (ct->basetype == NULL ||
 	    ct->subtype  == NULL ||
-	    cistrcmp(ct->basetype,"text") != 0 ||
-	    cistrcmp(ct->subtype,"plain") != 0) return 0; /* Not TEXT/PLAIN! */
+	    !CISTREQ(ct->basetype,"text") ||
+	    !CISTREQ(ct->subtype,"plain")) return 0; /* Not TEXT/PLAIN! */
 
 	if (ct->charset &&
-	    cistrncmp(ct->charset,"ISO-8859",8) != 0 &&
-	    cistrncmp(ct->charset,"KOI8",4)     != 0) return 0; /* Not ISO-* */
+	    !CISTREQN(ct->charset,"ISO-8859",8) &&
+	    !CISTREQN(ct->charset,"KOI8",4)    ) return 0; /* Not ISO-* */
 
 	if (ct->charset)
 	  free(ct->charset);
@@ -755,12 +1052,12 @@ downgrade_headers(rp, convertmode, verboselog)
 	char **CT   = NULL;
 	char **CTE  = NULL;
 	char **MIME = NULL;
-	/* char **receivedp = NULL; */
 	struct ct_data *ct;
-	int lines = 0;
-	int i;
+	int lines;
 	int is_textplain;
 	int newlen;
+	int outnum;
+	int oldidx, newidx;
 
 	if (*(rp->newmsgheadercvt) != NULL)
 	  return 0; /* Already converted ! */
@@ -769,7 +1066,8 @@ downgrade_headers(rp, convertmode, verboselog)
 
 	oldmsgheader = rp->newmsgheadercvt;
 
-	if (oldmsgheader)
+	lines = 0;
+	if (oldmsgheader) /* Count them! */
 	  while ((*oldmsgheader)[lines]) ++lines;
 
 	MIME = has_header(rp,"MIME-Version:");
@@ -782,21 +1080,22 @@ downgrade_headers(rp, convertmode, verboselog)
 	if (convertmode == _CONVERT_UNKNOWN) {
 	  /* We downgrade by changing it to Q-P as per RFC 1428/Appendix A */
 	  static const char *warning_lines[] = {
-"X-Warning: Original message contained 8-bit characters, however during",
-"\t   the SMTP transport session the receiving system did not announce",
-"\t   capability of receiving 8-bit SMTP (RFC 1651-1653), and as this",
-"\t   message does not have MIME headers (RFC 2045-2049) to enable",
-"\t   encoding change, we had very little choice.",
-"X-Warning: We ASSUME it is less harmful to add the MIME headers, and",
-"\t   convert the text to Quoted-Printable, than not to do so,",
-"\t   and to strip the message to 7-bits.. (RFC 1428 Appendix A)",
-"X-Warning: We don't know what character set the user used, thus we had to",
-"\t   write these MIME-headers with our local system default value.",
-"MIME-Version: 1.0",
-"Content-Transfer-Encoding: QUOTED-PRINTABLE",
+"X-Warning: Original message contained 8-bit characters, however during\n\
+\t   the SMTP transport session the receiving system did not announce\n\
+\t   capability of receiving 8-bit SMTP (RFC 1651-1653), and as this\n\
+\t   message does not have MIME headers (RFC 2045-2049) to enable\n\
+\t   encoding change, we had very little choice.\n\
+X-Warning: We ASSUME it is less harmful to add the MIME headers, and\n\
+\t   convert the text to Quoted-Printable, than not to do so,\n\
+\t   and to strip the message to 7-bits.. (RFC 1428 Appendix A)\n\
+X-Warning: We don't know what character set the user used, thus we had to\n\
+\t   write these MIME-headers with our local system default value.\n\
+MIME-Version: 1.0\n",
+"Content-Transfer-Encoding: QUOTED-PRINTABLE\n",
 NULL };
 
-	  char **newmsgheaders = (char**)malloc(sizeof(char**)*(lines+15));
+	  int outspc = lines + 8;
+	  char **newmsgheaders = (char**)malloc(sizeof(char**) * outspc);
 	  char *defcharset = getzenv("DEFCHARSET");
 	  char *newct;
 
@@ -804,35 +1103,39 @@ NULL };
 
 	  if (!defcharset)
 	    defcharset = "ISO-8859-1";
-#ifdef HAVE_ALLOCA
-	  newct = alloca(strlen(defcharset)+2+sizeof("Content-Type: TEXT/PLAIN; charset="));
-#else
 	  newct = malloc(strlen(defcharset)+2+sizeof("Content-Type: TEXT/PLAIN; charset="));
 	/* FIXME: malloc problem check ?? */
-#endif
 	  sprintf(newct,"Content-Type: TEXT/PLAIN; charset=%s",defcharset);
 
 	  if (!newmsgheaders) return 0; /* XX: Auch! */
 
-	  for (lines = 0; warning_lines[lines] != NULL; ++lines)
-	    newmsgheaders[lines] = strdup(warning_lines[lines]);
-	  newmsgheaders[lines++] = strdup(newct);
-#ifndef HAVE_ALLOCA
-	  free(newct);
-#endif
+	  outnum = 0;
+	  oldidx = 0;
+	  if ((*oldmsgheader)[0]) {
+	    newmsgheaders[outnum++] = (*oldmsgheader)[oldidx++];
+	  }
+
+	  for (newidx = 0; warning_lines[newidx]; ++newidx)
+	    newmsgheaders[outnum++] = strdup(warning_lines[newidx]);
+	  newmsgheaders[outnum++] = newct;
+
 	  if (CT)	/* XX: This CAN be wrong action for
 			       some esoteric SysV mailers.. */
 	    delete_header(rp,CT);
-	  /* These most propably won't happen, but the delete_header()
+	  /* These most probably won't happen, but the delete_header()
 	     does scram the pointers anyway.. */
-	  if (MIME)
+	  if (MIME) {
+	    MIME = has_header(rp,"MIME-Version:");
 	    delete_header(rp,MIME);
-	  if (CTE)
+	  }
+	  if (CTE) {
+	    CTE  = has_header(rp,cCTE);
 	    delete_header(rp,CTE);
+	  }
 
-	  for (i = 0; (*oldmsgheader)[i] != NULL; ++i)
-	    newmsgheaders[lines+i] = (*oldmsgheader)[i];
-	  newmsgheaders[lines+i] = NULL;
+	  for (; (*oldmsgheader)[oldidx]; ++oldidx)
+	    newmsgheaders[outnum++] = (*oldmsgheader)[oldidx];
+	  newmsgheaders[outnum] = NULL;
 
 	  free(*oldmsgheader); /* Free the old one.. */
 	  *oldmsgheader = newmsgheaders;
@@ -841,8 +1144,6 @@ NULL };
 
 	/* Now look for the  Content-Transfer-Encoding:  header */
 
-	/*receivedp = has_header(rp,"Received:");*/
-
 	if (CTE == NULL) return 0; /* No C-T-E: ??? */
 
 	/* strlen("Content-Transfer-Encoding: QUOTED-PRINTABLE") == 43
@@ -864,12 +1165,13 @@ NULL };
 	  return 0;
 	}
 
-	ct = parse_content_type(CT);
+	ct = parse_content_type(*CT);
 
 	is_textplain = (ct->basetype != NULL &&
+
 			ct->subtype  != NULL &&
-			cistrcmp(ct->basetype,"text") == 0 &&
-			cistrcmp(ct->subtype,"plain") == 0);
+			CISTREQ(ct->basetype,"text") &&
+			CISTREQ(ct->subtype,"plain"));
 
 	if (ct->charset && is_textplain &&
 	    (convertmode != _CONVERT_QP) &&
@@ -877,8 +1179,8 @@ NULL };
 	       inputs where the claimed charset(prefix) has
 	       all its charsets equal to US-ASCII in the
 	       low 128 characters. */
-	    (cistrncmp(ct->charset,"ISO-8859",8) == 0 ||
-	     cistrncmp(ct->charset,"KOI8",4)     == 0)) {
+	    (CISTREQN(ct->charset,"ISO-8859",8) ||
+	     CISTREQN(ct->charset,"KOI8",4)        )) {
 
 	  if (ct->charset)
 	    free(ct->charset);
@@ -912,66 +1214,50 @@ mime_received_convert(rp, convertstr)
 	char *convertstr;
 {
 	int convertlen = strlen(convertstr);
-	char *semic = NULL;
-	char **schdr = NULL;
-	int semicindex = -1;
+	int semicindex, receivedlen;
+	char *newreceived;
 
 	char **inhdr = *(rp->newmsgheadercvt);
-	char **hdro = NULL;
+	char *sc;
 
 	/* We have one advantage: The "Received:" header we
 	   want to fiddle with is the first one of them. */
 
-	if (!inhdr || cistrncmp(*inhdr,"Received:",9) != 0) {
+	if (!inhdr || !CISTREQN(*inhdr,"Received:",9)) {
 	  return 0; /* D'uh ??  Not 'Received:' ??? */
 	}
 
 	/* Look for the LAST semicolon in this Received: header.. */
 
-	do {
-	  char *sc = strrchr(*inhdr, ';');
-	  if (sc) {
-	    semic = sc;
-	    schdr = inhdr;
-	    semicindex = sc - *inhdr;
-	  }
-	  hdro = inhdr;
-	  ++inhdr;
-	} while (*inhdr && (**inhdr == ' ' || **inhdr == '\t'));
-
-	/* Now 'semic' is either set, or not; if set, 'schdr' points
-	   to the begin of the line where it is.
-	   If 'semic' is not set, 'hdro' will point to the last line
-	   of the 'Received:' header in question */
-
-	if (!semic) {
-	  schdr = hdro;
-	  semicindex = strlen(*schdr);
-	  semic = *schdr + semicindex;
+	sc = strrchr(*inhdr, ';');
+	if (sc) {
+	  semicindex = sc - *inhdr;
+	} else {
+	  semicindex = strlen(*inhdr);
+	  sc = *inhdr + semicindex;
 	}
 
-	{
-	  int receivedlen = strlen(*schdr);
-	  char *newreceived = malloc(receivedlen + convertlen + 1);
-	  int  semicindex;
+	receivedlen = strlen(*inhdr);
+	newreceived = malloc(receivedlen + convertlen + 1);
 
-	  if (!newreceived) return 0; /* Failed malloc.. */
+	if (!newreceived) return 0; /* Failed malloc.. */
 
-	  semicindex = semic - *schdr;
-	  memcpy(newreceived, *schdr, semicindex);
-	  memcpy(newreceived+semicindex, convertstr, convertlen);
-	  strcpy(newreceived+semicindex+convertlen, (*schdr) + semicindex);
-	  newreceived[receivedlen + convertlen] = '\0';
+	memcpy(newreceived, *inhdr, semicindex);
+	memcpy(newreceived+semicindex, convertstr, convertlen);
+	if (semicindex < receivedlen)
+	  memcpy(newreceived+semicindex+convertlen, (*inhdr) + semicindex,
+		 receivedlen - semicindex);
+	newreceived[receivedlen + convertlen] = '\0';
 
-	  ctlfree(rp->desc,*schdr);
-	  *schdr = newreceived;
+	ctlfree(rp->desc,*inhdr);
+	*inhdr = newreceived;
+
+	/* if (verboselog) {
+	   fprintf(verboselog,"Rewriting 'Received:' headers.\n");
+	   fprintf(verboselog,"The new line is: '%s'\n",*inhdr);
+	   }
+	*/
 
-	  /* if (verboselog) {
-	     fprintf(verboselog,"Rewriting 'Received:' headers.\n");
-	     fprintf(verboselog,"The new line is: '%s'\n",*hdro);
-	     }
-	  */
-	}
 	return 1;
 }
 
@@ -981,7 +1267,8 @@ qp_to_8bit(rp)
 	struct rcpt *rp;
 {
 	char **inhdr;
-	char *hdr, *p;
+	const char *p;
+	char *hdr;
 	char **CTE, **CT;
 	struct ct_data *ct;
 
@@ -995,17 +1282,18 @@ qp_to_8bit(rp)
 
 	if (!CTE || !CT) return 0; /* No C-T-E header! */
 
-	ct = parse_content_type(CT);
+	ct = parse_content_type(*CT);
 
 	if (ct->basetype == NULL ||
 	    ct->subtype  == NULL ||
-	    cistrcmp(ct->basetype,"text") != 0 ||
-	    cistrcmp(ct->subtype,"plain") != 0) return 0; /* Not TEXT/PLAIN! */
+	    !CISTREQ(ct->basetype,"text") ||
+	    !CISTREQ(ct->subtype,"plain") ) return 0; /* Not TEXT/PLAIN! */
 
 	hdr = *CTE;
 
 	p = hdr + 26;
-	while (*p == ' ' || *p == '\t') ++p;
+	p = skip_822linearcomments(p);
+
 	if (*p == 'Q' || *p == 'q') {
 	  if (strlen(hdr+26) >= 5)
 	    strcpy(hdr+26," 8BIT");
@@ -1017,7 +1305,7 @@ qp_to_8bit(rp)
 	  if (!mime_received_convert(rp," convert rfc822-to-8bit"))
 	    return 0;	/* "Received:" conversion failed! */
 
-	} /* else propably already decoded */
+	} /* else probably already decoded */
 
 	return 1;
 }
@@ -1032,7 +1320,8 @@ headers_need_mime2(rp)
 	while (inhdr && *inhdr) {
 	  u_char *hdr = (u_char *)*inhdr;
 	  for ( ; *hdr != 0 ; ++hdr)
-	    if (*hdr != '\t' && (*hdr < ' ' || *hdr > 126))
+	    if (*hdr != '\t' && *hdr != '\n' && *hdr != '\r'
+		&& (*hdr < ' ' || *hdr > 126))
 	      return 1;
 	  ++inhdr;
 	}
