@@ -24,6 +24,9 @@
 #include "ta.h"
 #include "libz.h"
 
+extern        int   wtttaidpid;
+extern CONVERTMODE  wttcvtmode;
+
 void
 tasyslog(rp,xdelay,wtthost,wttip,statstr,msg)
 struct rcpt *rp;
@@ -36,8 +39,8 @@ const char *msg;
   char spoolid[30];		/* Min. space: 6+8+1 chars	*/
   char delays[16], xdelays[16]; /* Min. space: 8+1 chars	*/
   time_t now;
-  static char *syslogflg = NULL;
-  char *t;
+  static const char *syslogflg = NULL;
+  const char *t;
 
   /* Syslogflag 'T' for classical format, and 't' for TAB-separated format */
 
@@ -103,5 +106,26 @@ const char *msg;
 	      wtthost, statstr, msg);
   }
 
+  if (wtttaidpid > 0) {
+    char *s = "UNKNOWN";
+    switch (wttcvtmode) {
+    case _CONVERT_NONE:
+      s = "NONE";
+      break;
+    case _CONVERT_MULTIPARTQP:
+      s = "MPQP";
+      break;
+    case _CONVERT_QP:
+      s = "QP";
+      break;
+    case _CONVERT_8BIT:
+      s = "8BIT";
+      break;
+    case _CONVERT_UNKNOWN:
+      break;
+    }
+    sprintf(linebuf + strlen(linebuf), " cvt=%s", s);
+  }
+
   zsyslog((LOG_INFO, "%s", linebuf));
 }
