@@ -1,7 +1,7 @@
 /*
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
- *	Copyright 1992-1997 Matti Aarnio -- MIME processing et.al.
+ *	Copyright 1992-2000 Matti Aarnio -- MIME processing et.al.
  */
 
 /* History:
@@ -30,10 +30,9 @@
 
 #define DefCharset "ISO-8859-1"
 
-#include "hostenv.h"
+#include "mailer.h"
 #include <ctype.h>
 #include <errno.h>
-#include <pwd.h>
 #include <sysexits.h>
 #include <sys/param.h>
 #include <fcntl.h>
@@ -230,7 +229,7 @@ const char *maildirs[] = {
 
 #ifdef CHECK_MB_SIZE
 extern int checkmbsize(const char *uname, const char *host, const char *user,
-		       size_t cursize, struct passwd *pw);
+		       size_t cursize, struct Zpasswd *pw);
 #endif
 
 #if	defined(HAVE_SOCKET)
@@ -319,7 +318,7 @@ int  do_xuidl = 0;		/* Store our own  X-UIDL: header to allow
 				   counter..  See RFC 2060 for IMAP4. */
 
 extern int fmtmbox __((char *, int, const char *, const char *, \
-			const struct passwd *));
+			const struct Zpasswd *));
 
 extern RETSIGTYPE wantout __((int));
 extern int optind;
@@ -327,19 +326,16 @@ extern char *optarg;
 extern void biff __((const char *, const char *, long));
 extern void rbiff __((struct biffer *));
 extern void prversion __((const char *));
-#if 0
-extern struct passwd *getpwnam __((const char *));
-extern struct passwd *getpwuid __((uid_t));
-#endif
+
 extern int setupuidgid __((struct rcpt *, int, int));
 extern int createfile __((struct rcpt *, const char *, int, int));
 extern int exstat __((struct rcpt *, const char *, struct stat *, int (*)(const char*, struct stat*) ));
-extern int creatembox __((struct rcpt *, const char *, char **, uid_t*, gid_t*, struct passwd *));
-extern char *exists __((const char *, const char *, struct passwd *, struct rcpt *));
+extern int creatembox __((struct rcpt *, const char *, char **, uid_t*, gid_t*, struct Zpasswd *));
+extern char *exists __((const char *, const char *, struct Zpasswd *, struct rcpt *));
 extern void setrootuid __((struct rcpt *));
 extern void process __((struct ctldesc *dp));
 extern void deliver __((struct ctldesc *dp, struct rcpt *rp, const char *userbuf, const char *timestring));
-extern Sfio_t *putmail __((struct ctldesc *dp, struct rcpt *rp, int fdmail, const char *fdopmode, const char *timestring, const char *file));
+extern Sfio_t *putmail __((struct ctldesc *dp, struct rcpt *rp, int fdmail, const char *fdopmode, const char *timestring, const char *file, uid_t));
 extern int appendlet __((struct ctldesc *dp, struct rcpt *rp, struct writestate *WS, const char *file, int ismime));
 extern char **environ;
 extern int writebuf __((struct writestate *, const char *buf, int len));
@@ -368,9 +364,18 @@ extern void * emalloc __((size_t));
 #else
 struct conshell *envarlist = NULL;
 #endif
-extern int stickymem;	/* for strsave() */
 int	D_alloc = 0;
 
+static int zsfsetfd(fp, fd)
+     Sfio_t *fp;
+     int fd;
+{
+  /* This is *NOT* the SFIO's sfsetfd() -- we do no sfsync() at any point.. */
+  fp->file = fd;
+  return fd;
+}
+
+
 static void decodeXtext __((Sfio_t *, const char *));
 
 #if defined(HAVE_SOCKET) && defined(HAVE_PROTOCOLS_RWHOD_H)
@@ -406,6 +411,7 @@ main(argc, argv)
 	const char *argv[];
 {
 	char *s;
+	const char *cs;
 	int c, errflg, fd;
 	char *host = NULL;	/* .. and what is my host ? */
 	int matchhost = 0;
@@ -439,9 +445,9 @@ main(argc, argv)
 	else
 	  ++progname;
 
-	s = getzenv("MAILBOX");
-	if (s != NULL) {
-	  maildirs[0] = s;
+	cs = getzenv("MAILBOX");
+	if (cs != NULL) {
+	  maildirs[0] = cs;
 	  maildirs[1] = NULL;
 	}
 
@@ -705,10 +711,10 @@ process(dp)
 
 	hdr = has_header(rp,"Content-Type:");
 	if (hdr)
-	  CT = parse_content_type(hdr);
+	  CT = parse_content_type(*hdr);
 	hdr = has_header(rp,"Content-Transfer-Encoding:");
 	if (hdr)
-	  CTE = parse_content_encoding(hdr);
+	  CTE = parse_content_encoding(*hdr);
 	if (CT) {
 	  if (CT->basetype == NULL ||
 	      CT->subtype  == NULL ||
@@ -791,12 +797,12 @@ process(dp)
 }
 
 /*
- * propably_x400() -- some heuristics to see if this is likely
+ * probably_x400() -- some heuristics to see if this is likely
  *		      a mis-written X.400 address
  */
-int propably_x400 __((const char *));
+int probably_x400 __((const char *));
 int
-propably_x400(addr)
+probably_x400(addr)
 	const char *addr;
 {
 	int slashes = 0;
@@ -838,16 +844,18 @@ struct rcpt *rp;
 const char *file;
 int iuid;
 {
-#ifdef	HAVE_MAILLOCK_H
+#ifdef	HAVE_MAILLOCK
 	const char *maillockuser;
-	struct passwd *pw;
+	struct Zpasswd *pw;
 	int i;
 	char errbuf[30];
 	const char *cp;
 	uid_t uid = (uid_t) iuid;
 
+	pw = zgetpwuid(uid);
+
 	if (*(rp->addr->user) == TO_FILE) {
-	  if ((pw = getpwuid(uid)) == NULL) {
+	  if (pw == NULL) {
 	    notaryreport(file,"failed",
 			 "5.2.1 (Target file has no known owner)",
 			 "x-local; 510 (Target file has no known owner)");
@@ -856,9 +864,22 @@ int iuid;
 	    return 1;
 	  }
 	  maillockuser = pw->pw_name;
-	}
-	else
+	} else {
+	  char *at, *plus;
 	  maillockuser = rp->addr->user;
+	  at = strchr(maillockuser, '@');
+	  if (at) *at = 0;
+	  plus = strchr(maillockuser, '+');
+	  if (plus) *plus = 0;
+
+	  pw = zgetpwnam(maillockuser);
+
+	  if (plus) *plus = '+';
+	  if (at) *at = '@';
+
+	  if (pw)
+	    maillockuser = pw->pw_name;
+	}
 
 	i = maillock((char*)maillockuser, 2); /* Sigh.. proto vs. man-page */
 	switch (i) {
@@ -902,7 +923,7 @@ int iuid;
 	  return 1;
 	}
 	havemaillock = 1;
-#endif	/* HAVE_MAILLOCK_H */
+#endif	/* HAVE_MAILLOCK */
 #ifdef  HAVE_DOTLOCK
 	havedotlock = (dotlock(file) == 0);
 	if (!havedotlock) {
@@ -1038,7 +1059,7 @@ deliver(dp, rp, usernam, timestring)
 #endif
 #endif
 	const char *unam = usernam;
-	struct passwd *pw = NULL;
+	struct Zpasswd *pw = NULL;
 	time_t starttime;
 
 	time(&starttime);
@@ -1088,15 +1109,15 @@ deliver(dp, rp, usernam, timestring)
 	     to the actual device.. So lets just use that magic
 	     name and create a FAST "write" to  /dev/null..  */
 	  if (strcmp(usernam,"/dev/null") == 0) {
-	    notaryreport(rp->addr->user,"delivery",
-			 "2.2.0 (delivery successfull)",
+	    notaryreport(rp->addr->user,"delivered",
+			 "2.2.0 (delivered successfully)",
 			 "x-local; 250 (Delivered successfully)");
 	    DIAGNOSTIC(rp, usernam, EX_OK, "Ok", 0);
 	    return;
 	  }
 
 	  if (uid == nobody) {
-	    if (propably_x400(usernam)) {
+	    if (probably_x400(usernam)) {
 
 	      if (verboselog)
 		fprintf(verboselog,
@@ -1137,7 +1158,9 @@ deliver(dp, rp, usernam, timestring)
 	      return;
 	    }
 #ifdef HAVE_FSYNC
-	    fsync(fdmail);
+	    while (fsync(fdmail) < 0)
+	      if (errno != EINTR && errno != EAGAIN)
+		break;
 #endif
 	    close(fdmail);
 	  }
@@ -1153,15 +1176,14 @@ deliver(dp, rp, usernam, timestring)
 			 needed for multi-recipient processing ? */
 #endif
 	  unam = usernam;
-	  errno = 0;
-	  pw = getpwnam(usernam);
+
+	  pw = zgetpwnam(usernam);
 	  if (pw == NULL) {
 
 	    /* No match as is ?  Lowercasify, and try again! */
 	    strlower((char*)usernam);
 
-	    errno = 0;
-	    pw = getpwnam(usernam);
+	    pw = zgetpwnam(usernam);
 	    if (pw == NULL) {
 	      if (plus) *plus = '+';
 
@@ -1183,33 +1205,23 @@ deliver(dp, rp, usernam, timestring)
 		 problem, which time will solve (as with system
 		 operator taking some action) ?  */
 
-	      /* Many systems seem to use this in otherwise
-		 fine lookup -- to mark lack of data */
-	      if (errno == ENOENT)
-		errno = 0;
-#ifdef __osf__
-	      /* ... and OSF/1 *must* be different, of course ...
-		 (I just wonder what AIX does) */
-	      if (errno == EINVAL)
-		errno = 0;
-#endif
-	      if (errno != 0) { /* getpwnam() failed for some other
+	      if (errno != 0) { /* zgetpwnam() failed for some other
 				   reason than simply not finding the
 				   given user... */
 		int err = errno;
 
 		if (verboselog)
 		  fprintf(verboselog,
-			"   getpwnam(\"%s\") failed (%d)\n", usernam, errno);
+			"   zgetpwnam(\"%s\") failed (%d)\n", usernam, errno);
 
 		notaryreport(rp->addr->user,"failed",
 			     "5.3.0 (Error getting user identity)",
 			     "x-local; 550 (Error getting user identity)");
 		DIAGNOSTIC3(rp, usernam, EX_TEMPFAIL,
-			   "getpwnam for user \"%s\" failed; errno=%d",
+			   "zgetpwnam for user \"%s\" failed; errno=%d",
 			   usernam, err);
 
-	      } else if (propably_x400(usernam)) {
+	      } else if (probably_x400(usernam)) {
 
 		if (verboselog)
 		  fprintf(verboselog,
@@ -1262,7 +1274,7 @@ deliver(dp, rp, usernam, timestring)
 	  
 	  if (!hasdir) {	/* No directory ?? */
 
-	    char *mailbox = getzenv("MAILBOX");
+	    const char *mailbox = getzenv("MAILBOX");
 	    notaryreport(rp->addr->user,"failed",
 			 "5.3.5 (System mailbox configuration is wrong, we are in deep trouble..)",
 			 "x-local; 566 (System mailbox configuration is wrong!  No such directory!  Aargh!)");
@@ -1312,8 +1324,8 @@ deliver(dp, rp, usernam, timestring)
 	   to the actual device.. So lets just use that magic
 	   name and create a FAST "write" to  /dev/null..  */
 	if (strcmp(file,"/dev/null") == 0) {
-	  notaryreport(rp->addr->user,"delivery",
-		       "2.2.0 (delivery successfull)",
+	  notaryreport(rp->addr->user,"delivered",
+		       "2.2.0 (delivered successfully)",
 		       "x-local; 250 (Delivered successfully)");
 	  DIAGNOSTIC(rp, usernam, EX_OK, "Ok", 0);
 	  return;
@@ -1345,7 +1357,7 @@ deliver(dp, rp, usernam, timestring)
 	if (1) {
 	  extern  int checkmbsize __((const char *uname,
 				      const char *host, const char *user,
-				      size_t cursize, struct passwd *pw));
+				      size_t cursize, struct Zpasswd *pw));
 
 	  /* external procedure checkmbsize() accepts user name, "host"
 	     name as on routing result, "user" part of routed data,
@@ -1459,7 +1471,7 @@ deliver(dp, rp, usernam, timestring)
 			  starttime, timestring );
 	  } else {
 	    /* This happens only ONCE per recipient, if ever */
-	    notaryreport(rp->addr->user,"delivery",
+	    notaryreport(rp->addr->user,"delivered",
 			 "2.2.0 (Discarded successfully)",
 			 "x-local; 250 (Discarded successfully)");
 	    DIAGNOSTIC(rp, usernam, EX_OK, "Ok", 0);
@@ -1632,11 +1644,11 @@ void store_to_file(dp,rp,file,ismbox,usernam,st,uid,
 		case '"':
 		  break;
 		case '.':
-#ifdef	HAVE_MAILLOCK_H
+#ifdef	HAVE_MAILLOCK
 		  if (havemaillock && ismbox)
 		    mailunlock();
 		  havemaillock = 0;
-#endif	/* HAVE_MAILLOCK_H */
+#endif	/* HAVE_MAILLOCK */
 #ifdef	HAVE_DOTLOCK
 		  if (ismbox)
 		    dotunlock(file);
@@ -1684,7 +1696,7 @@ void store_to_file(dp,rp,file,ismbox,usernam,st,uid,
 	  nbp->offset = eofindex;
 #endif	/* BIFF || RBIFF */
 
-	fp = putmail(dp, rp, fdmail, "a+", timestring, file);
+	fp = putmail(dp, rp, fdmail, "a+", timestring, file, uid);
 	
 	if (S_ISREG(st->st_mode)) {
 
@@ -1700,11 +1712,11 @@ void store_to_file(dp,rp,file,ismbox,usernam,st,uid,
 	    case '"':
 	      break;
 	    case '.':
-#ifdef	HAVE_MAILLOCK_H
+#ifdef	HAVE_MAILLOCK
 	      if (ismbox && havemaillock)
 		mailunlock();
 	      havemaillock = 0;
-#endif	/* HAVE_MAILLOCK_H */
+#endif	/* HAVE_MAILLOCK */
 #ifdef	HAVE_DOTLOCK
 	      if (ismbox)
 		dotunlock(file);
@@ -1744,7 +1756,7 @@ void store_to_file(dp,rp,file,ismbox,usernam,st,uid,
 	close(fdmail);
 	if (fp != NULL) { /* Dummy marker! */
 	  notary_setxdelay((int)(endtime-starttime));
-	  notaryreport(rp->addr->user,"delivery",
+	  notaryreport(rp->addr->user,"delivered",
 		       "2.2.0 (Delivered successfully)",
 		       "x-local; 250 (Delivered successfully)");
 	  DIAGNOSTIC(rp, usernam, EX_OK, "Ok", 0);
@@ -1816,11 +1828,12 @@ if (verboselog)
 
 
 Sfio_t *
-putmail(dp, rp, fdmail, fdopmode, timestring, file)
+putmail(dp, rp, fdmail, fdopmode, timestring, file, uid)
      struct ctldesc *dp;
      struct rcpt *rp;
      int fdmail;
      const char *fdopmode, *timestring, *file;
+     uid_t uid;
 {
 	int len, rc, mw=0;
 	Sfio_t *fp;
@@ -1836,7 +1849,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 
 	fstat(fdmail, &st);
 
-	fp = sfnew(NULL, NULL, 16*1024, fdmail, SF_WRITE|SF_APPEND);
+	fp = sfnew(NULL, NULL, 64*1024, fdmail, SF_READ|SF_WRITE|SF_APPEND);
 	if (fp == NULL) {
 	  notaryreport(NULL,NULL,NULL,NULL);
 	  DIAGNOSTIC3(rp, file, EX_TEMPFAIL, "cannot fdopen(%d,\"%s\")",
@@ -1899,9 +1912,6 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    strcmp(rp->addr->link->channel, "error") == 0)
 	  fromuser = "";
 
-	if (*fromuser == 0)
-	  fromuser = "MAILER-DAEMON";
-
 	do {
 	  hdrs = has_header(rp,"Return-Path:");
 	  if (hdrs) delete_header(rp,hdrs);
@@ -1909,6 +1919,10 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 
 	append_header(rp,"Return-Path: <%.999s>", fromuser);
 
+
+	if (*fromuser == 0)
+	  fromuser = "MAILER-DAEMON";
+
 	hdrs = has_header(rp,"To:");
 	if (!hdrs) {
 	  /* No "To:" -header ?  Rewrite possible "Apparently-To:" header! */
@@ -1922,6 +1936,11 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  append_header(rp,"Apparently-To: <%.999s>", rp->addr->link->user);
 	}
 
+	do {
+	  hdrs = has_header(rp,"X-Envelope-To:");
+	  if (hdrs) delete_header(rp,hdrs);
+	} while (hdrs);
+
 	do {
 	  hdrs = has_header(rp,"X-Orcpt:");
 	  if (hdrs) delete_header(rp,hdrs);
@@ -1944,6 +1963,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    || swriteheaders(rp, fp, "\n", convert_qp, 0, NULL) < 0)
 	  failed = 1;
 
+	if (!failed)
+	  sfprintf(fp, "X-Envelope-To: <%s> (uid %d)\n", rp->addr->user, uid);
+
 	if (!failed && rp->orcpt) {
 	  sfprintf(fp, "X-Orcpt: ");
 	  decodeXtext(fp, rp->orcpt);
@@ -1985,10 +2007,14 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    break;
 	  }
 	  if (eofindex >= 0)
-	    ftruncate(sffileno(fp), (u_long)eofindex);
-#endif
+	    while (ftruncate(sffileno(fp), (u_long)eofindex) < 0)
+	      if (errno != EINTR && errno != EAGAIN)
+		break;
+#endif /* HAVE_FTRUNCATE */
 #ifdef HAVE_FSYNC
-	  fsync(sffileno(fp));
+	  while (fsync(sffileno(fp)) < 0)
+	    if (errno != EINTR && errno != EAGAIN)
+	      break;
 #endif
 	  sfclose(fp);
 	  fp = NULL;
@@ -2012,10 +2038,14 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    /* XX: should I really do this? */
 	    sfsync(fp);
 	    if (eofindex >= 0)
-	      ftruncate(sffileno(fp), (off_t)eofindex);
-#endif
+	      while (ftruncate(sffileno(fp), (off_t)eofindex) < 0)
+		if (errno != EINTR && errno != EAGAIN)
+		  break;
+#endif /* HAVE_FTRUNCATE */
 #ifdef HAVE_FSYNC
-	    fsync(sffileno(fp));
+	    while (fsync(sffileno(fp)) < 0)
+	      if (errno != EINTR && errno != EAGAIN)
+		break;
 #endif
 	    sfclose(fp);
 	    eofindex = -1;
@@ -2023,6 +2053,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    goto time_reset;
 	  }
 
+if (verboselog)
+  fprintf(verboselog," end of putmail(file='%s'), topipe=%d\n",file,topipe);
+
 	if (!topipe) {
 	  /*
 	   * Ok, we are NOT writing to a pipe, and thus we can do
@@ -2039,9 +2072,13 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	   * login etc. can distinguish new mail from old.
 	   * The mtime will be set to now by the following write() calls.
 	   */
-	  sfseek(fp, (off_t)-2, SEEK_END);
+	  sfseek(fp, (Sfoff_t)-2LL, SEEK_END);
 	  len = sfread(fp, buf, 2);
-	  sfseek(fp, (off_t)0, SEEK_END);	/* to end of file, again */
+	  sfseek(fp, (Sfoff_t)0,    SEEK_END);
+	  /* to end of file, again */
+
+if (verboselog)
+  fprintf(verboselog," .. EOF read did yield %d bytes\n", len);
 
 	  if (len == 1 || len == 2) {
 	    int err;
@@ -2050,6 +2087,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    err = (len > 0 && (sfwrite(fp, "\n\n", len) != len));
 	    sfsync(fp);
 
+if (verboselog)
+  fprintf(verboselog," .. wrote %d newlines to the end\n", len);
+
 	    if (!err) err = sferror(fp);
 	    if (err) {
 	      notaryreport(NULL,NULL,NULL,NULL);
@@ -2059,10 +2099,14 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	      /* XX: should I really do this? */
 	      sfsync(fp);
 	      if (eofindex >= 0)
-		ftruncate(sffileno(fp), (off_t)eofindex);
+		while (ftruncate(sffileno(fp), (off_t)eofindex) < 0)
+		  if (errno != EINTR && errno != EAGAIN)
+		    break;
 #endif /* HAVE_FTRUNCATE */
 #ifdef HAVE_FSYNC
-	      fsync(sffileno(fp));
+	      while (fsync(sffileno(fp)) < 0)
+		if (errno != EINTR && errno != EAGAIN)
+		  break;
 #endif
 	      sfclose(fp);
 	      fp = NULL;
@@ -2086,7 +2130,13 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  goto write_failure;
 	}
 #ifdef HAVE_FSYNC
-	fsync(fdmail);
+	if (mmdf_mode <= 1) {
+	  while (fsync(fdmail) < 0) {
+	    if (errno == EINTR || errno == EAGAIN)
+	      continue;
+	    goto write_failure;
+	  }
+	}
 #endif
 
 
@@ -2130,7 +2180,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	}
 
 
-	sfsetfd(fp, -1);
+	zsfsetfd(fp, -1);
 	sfclose(fp);
 
 	return fp; /* Dummy marker! */
@@ -2151,7 +2201,7 @@ program(dp, rp, cmdbuf, user, timestring, uid)
 	const char *s;
 	char buf[8192], *cp, *cpe;
 	int status;
-	struct passwd *pw;
+	struct Zpasswd *pw;
 	Sfio_t *errfp;
 	Sfio_t *fp;
 	time_t starttime, endtime;
@@ -2181,7 +2231,7 @@ program(dp, rp, cmdbuf, user, timestring, uid)
 	  cp += strlen(cp) + 1;
 	}
 
-	pw = getpwuid(uid);
+	pw = zgetpwuid(uid);
 	if (pw == NULL) {
 
 	  if (verboselog) {
@@ -2240,6 +2290,18 @@ program(dp, rp, cmdbuf, user, timestring, uid)
 	  env[envi++] = cp;
 	  cp += strlen(cp) + 1;
 	}
+	
+	env[envi++] = cp;
+	strcpy(cp, "MSGSPOOLID="); cp += 11;
+	taspoolid(cp, rp->desc->msgmtime, rp->desc->msginonumber);
+	cp += strlen(cp) + 1;
+
+	if (rp->desc->msgfile) {
+	  /* Put also the message-id of the message into variables. */
+	  env[envi++] = cp;
+	  sprintf(cp, "MESSAGEID=%.199s", rp->desc->msgfile);
+	  cp += strlen(cp) + 1;
+	}
 
 	if (verboselog)
 	  fprintf(verboselog,"To run a pipe with uid=%d gid=%d cmd='%s'\n",
@@ -2328,7 +2390,7 @@ program(dp, rp, cmdbuf, user, timestring, uid)
 
 	  } else {
 
-	    /* Duh, propably something like:
+	    /* Duh, probably something like:
 	       "|IFS=' '&&.... "
 	    */
 
@@ -2373,7 +2435,7 @@ program(dp, rp, cmdbuf, user, timestring, uid)
 	/* write the message */
 	mmdf_mode += 2;
 	eofindex = -1; /* NOT truncatable! */
-	fp = putmail(dp, rp, out[1], "a", timestring, cmdbuf);
+	fp = putmail(dp, rp, out[1], "a", timestring, cmdbuf, uid);
 	/* ``fp'' is dummy marker */
 	mmdf_mode -= 2;
 	if (fp == NULL) {
@@ -2386,7 +2448,7 @@ program(dp, rp, cmdbuf, user, timestring, uid)
 	/* read any messages from its stdout/err on in[0] */
 	/* ... having forked and set up the pipe, we quickly continue */
 	buf[sizeof(buf)-100] = 0; /* Chop it just to make sure */
-	if (cfgets(buf, (sizeof buf) - 100, errfp) < 0)
+	if (csfgets(buf, (sizeof buf) - 100, errfp) < 0)
 		buf[0] = '\0';
 	else if ((cp = strchr(buf, '\n')) != NULL)
 		*cp = '\0';
@@ -2426,7 +2488,7 @@ program(dp, rp, cmdbuf, user, timestring, uid)
 	time(&endtime);
 	notary_setxdelay((int)(endtime-starttime));
 	if (rc == EX_OK) {
-	  notaryreport(NULL,"delivery",
+	  notaryreport(NULL,"delivered",
 		       "2.2.0 (Delivered successfully)",
 		       "x-local; 250 (Delivered successfully)");
 	} else {
@@ -2506,7 +2568,7 @@ creatembox(rp, uname, filep, uid, gid, pw)
 	char **filep;
 	uid_t *uid;
 	gid_t *gid;
-	struct passwd *pw;
+	struct Zpasswd *pw;
 {
 	const char **maild;
 	int fd = -1;
@@ -2554,13 +2616,13 @@ creatembox(rp, uname, filep, uid, gid, pw)
 	      struct stat st;
 #ifdef	HAVE_UTIME
 	      struct utimbuf tv;
-	      stat(*filep,&st); /* This by all propability will not fail.. */
+	      stat(*filep,&st); /* This by all probability will not fail.. */
 	      tv.actime  = 0;	/* never read */
 	      tv.modtime = st.st_mtime;
 	      utime(*filep, &tv);
 #else
 	      struct timeval tv[2];
-	      stat(*filep,&st); /* This by all propability will not fail.. */
+	      stat(*filep,&st); /* This by all probability will not fail.. */
 	      tv[0].tv_sec = 0; /* never read */
 	      tv[1].tv_sec = st.st_mtime;
 	      tv[0].tv_usec = tv[1].tv_usec = 0;
@@ -2569,7 +2631,7 @@ creatembox(rp, uname, filep, uid, gid, pw)
 	    }
 	    return 1;
 	  }
-	  if (errno == EEXIST) { /* It exists -- propably a race between
+	  if (errno == EEXIST) { /* It exists -- probably a race between
 				    two file creators caused this */
 	    
 	    return 1;
@@ -2699,7 +2761,7 @@ setupuidgid(rp, uid, gid)
 
 	if (gid == -3) {
 	  /* MAGIC! Ask GID of the UID of 'uid' */
-	  struct passwd *pw = getpwuid(uid);
+	  struct Zpasswd *pw = zgetpwuid(uid);
 	  if (pw != NULL)
 	    gid = pw->pw_gid;
 	}
@@ -2735,7 +2797,7 @@ char *
 exists(maildir, uname, pw, rp)
 	const char *maildir;
 	const char *uname;
-	struct passwd *pw;
+	struct Zpasswd *pw;
 	struct rcpt *rp;
 {
 	char *file, *s;
@@ -2847,7 +2909,7 @@ appendlet(dp, rp, WS, file, ismime)
 	  lseek(mfd, (off_t)dp->msgbodyoffset, SEEK_SET);
 	  mfp = sfnew(NULL, NULL, 16*1024, mfd, SF_READ|SF_WHOLE);
 
-#define MFPCLOSE sfsetfd(mfp,-1); sfclose(mfp);
+#define MFPCLOSE zsfsetfd(mfp,-1); sfclose(mfp);
 
 	  /* we are assuming to be positioned properly
 	     at the start of the message body */
@@ -2855,7 +2917,7 @@ appendlet(dp, rp, WS, file, ismime)
 	  /* We really can't use the 'let_buffer' cache here */
 	  readalready = 0;
 	  i = 0;
-	  while ((i = cfgets(let_buffer, sizeof(let_buffer), mfp)) != EOF) {
+	  while ((i = csfgets(let_buffer, sizeof(let_buffer), mfp)) != EOF) {
 	    /* It MAY be malformed -- if it has a BUFSIZ length
 	       line in it, IT CAN'T BE MIME  :-/		*/
 	    if (i == sizeof(let_buffer) &&
@@ -3317,7 +3379,7 @@ find_return_receipt_hdr (rp)
     const char **ptr, *hdr;
 
     for (ptr = *((const char ***)rp->newmsgheader); *ptr != NULL; ptr++) {
-        if (cistrncmp(*ptr, "Return-Receipt-To:", 18) == 0)
+        if (CISTREQN(*ptr, "Return-Receipt-To:", 18))
             break;
     }
 
@@ -3385,13 +3447,19 @@ return_receipt (dp, retrecptaddr, uidstr)
 	struct rcpt *rp;
 	int n;
 	char boundarystr[400];
-	struct passwd *pw;
+	struct Zpasswd *pw;
 	int uid;
 	struct stat stb;
 	const char *username = "unknown";
+	char *retaddr = strsave(retrecptaddr);
+	char *s;
+
+	while ((s = strchr(retaddr, '\n'))) *s = ' ';
+	while ((s = strchr(retaddr, '\r'))) *s = ' ';
+	while ((s = strchr(retaddr, '\t'))) *s = ' ';
 
 	uid = atoi(uidstr);
-	pw = getpwuid(uid);
+	pw = zgetpwuid(uid);
 	if (pw)
 	  username = pw->pw_name;
 
@@ -3408,13 +3476,15 @@ return_receipt (dp, retrecptaddr, uidstr)
 
 	/* copy To: from return-receipt address */
 	sfprintf(mfp, "todsn NOTIFY=NEVER ORCPT=rfc822;");
-	encodeXtext(mfp, retrecptaddr);
-	sfprintf(mfp, "\nto %s\n",retrecptaddr);
+	encodeXtext(mfp, retaddr);
+	sfprintf(mfp, "\nto %s\n",retaddr);
 	sfprintf(mfp, "env-end\n");
-	sfprintf(mfp, "To: %s\n", retrecptaddr);
+	sfprintf(mfp, "To: %s\n", retaddr);
 	sfprintf(mfp, "From: Automatically on behalf of the user <%s>\n",
 		 username);
 
+	free(retaddr);
+
 	/* copy error message file itself */
 	mailshare = getzenv("MAILSHARE");
 	if (mailshare == NULL)
@@ -3441,7 +3511,7 @@ return_receipt (dp, retrecptaddr, uidstr)
 	if (efp != NULL) {
 	  int inhdr = 1;
 	  buf[sizeof(buf)-1] = 0;
-	  while (cfgets(buf,sizeof(buf)-1,efp) >= 0) {
+	  while (csfgets(buf,sizeof(buf)-1,efp) >= 0) {
 	    if (strncmp(buf,"HDR",3)==0) {
 	      sfprintf(mfp, "%s", buf+4);
 	    } else if (strncmp(buf,"SUB",3)==0) {
@@ -3497,8 +3567,8 @@ return_receipt (dp, retrecptaddr, uidstr)
 	    sfprintf(mfp, "\n");
 	  }
 	  sfprintf(mfp, "Final-Recipient: X-LOCAL;%s\n", rp->addr->user);
-	  sfprintf(mfp, "Action: delivery\n");
-	  sfprintf(mfp, "Status: 2.2.0\n");
+	  sfprintf(mfp, "Action: delivered\n");
+	  sfprintf(mfp, "Status: 2.2.0 (delivered successfully)\n");
 	  sfprintf(mfp, "Diagnostic-Code: smtp; 250 ('%s' delivered)\n", rp->addr->user );
 	  sfprintf(mfp, "\n");
 	}
