@@ -32,17 +32,23 @@ getmxrr(SS, host, mx, maxmx, depth)
 
 	h_errno = 0;
 
+#ifndef TEST
 	notary_setwtt  (NULL);
 	notary_setwttip(NULL);
+#endif
 
-	if (depth == 0)
+	if (depth == 0) {
 	  SS->mxcount = 0;
+	  sprintf(SS->remotemsg, "-- Lookup of MX/A for '%.200s'", host);
+	}
 
 	if (depth > 3) {
 	  sprintf(SS->remotemsg,"smtp; 500 (DNS: Recursive CNAME on '%.200s')",host);
 	  time(&endtime);
+#ifndef TEST
 	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.3 (Recursive DNS CNAME)",SS->remotemsg);
+#endif
 	  fprintf(stderr, "%s\n", SS->remotemsg);
 	  return EX_NOHOST;
 	}
@@ -58,8 +64,10 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    fprintf(SS->verboselog,"  %s\n", SS->remotemsg);
 
 	  time(&endtime);
+#ifndef TEST
 	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.3 (DNS-failure)",SS->remotemsg);
+#endif
 	  return EX_SOFTWARE;
 	}
 	n = res_send((void*)&qbuf, qlen, (void*)&answer, sizeof answer);
@@ -71,8 +79,10 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    fprintf(SS->verboselog,"  %s\n", SS->remotemsg);
 
 	  time(&endtime);
+#ifndef TEST
 	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.3 (DNS-failure)",SS->remotemsg);
+#endif
 	  return EX_TEMPFAIL;
 	}
 
@@ -98,6 +108,10 @@ getmxrr(SS, host, mx, maxmx, depth)
 	  fprintf(SS->verboselog, "\n");
 	}
 
+	rmsgappend(SS, 1,
+		   "\r-> DNSreply: len=%d rcode=%d qd=%d an=%d ns=%d ar=%d",
+		   n, hp->rcode, qdcount, ancount, nscount, arcount);
+
 	if (hp->rcode != NOERROR || ancount == 0) {
 	  switch (hp->rcode) {
 	  case NXDOMAIN:
@@ -110,16 +124,20 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog," NXDOMAIN %s\n", SS->remotemsg);
 	    endtime = now;
+#ifndef TEST
 	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",SS->remotemsg);
+#endif
 	    return EX_NOHOST;
 	  case SERVFAIL:
 	    sprintf(SS->remotemsg, "smtp; 500 (DNS: server failure: %.200s)", host);
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog," SERVFAIL %s\n", SS->remotemsg);
 	    endtime = now;
+#ifndef TEST
 	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",SS->remotemsg);
+#endif
 	    return EX_TEMPFAIL;
 	  case NOERROR:
 	    mx[0].host = NULL;
@@ -132,16 +150,20 @@ getmxrr(SS, host, mx, maxmx, depth)
 	      fprintf(SS->verboselog," FORMERR/NOTIMP/REFUSED(%d) %s\n",
 		      hp->rcode, SS->remotemsg);
 	    endtime = now;
+#ifndef TEST
 	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",SS->remotemsg);
+#endif
 	    return EX_NOPERM;
 	  }
 	  sprintf(SS->remotemsg, "smtp; 500 (DNS: unknown error, MX info unavailable: %.200s)", host);
 	  if (SS->verboselog)
 	    fprintf(SS->verboselog,"  %s\n", SS->remotemsg);
 	  endtime = now;
+#ifndef TEST
 	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",SS->remotemsg);
+#endif
 
 	  if (had_eai_again)
 	    return EX_TEMPFAIL;
@@ -214,6 +236,8 @@ getmxrr(SS, host, mx, maxmx, depth)
 	  if (SS->verboselog)
 	    fprintf(SS->verboselog, " -> (%lds) MX[%d] pref=%d host=%s\n",
 		    mx[nmx].expiry - now, nmx, mx[nmx].pref, buf);
+	  rmsgappend(SS, 1, "\r-> %lds MX[%d] p=%d '%s'",
+		     mx[nmx].expiry - now, nmx, mx[nmx].pref, mx[nmx].host);
 	  mxtype[nmx] = 0;
 	  ++nmx;
 	  --ancount;
@@ -266,12 +290,16 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    /* Yes, it is TRUNCATED reply!   Must retry with e.g.
 	       by using TCP! */
 	    /* FIXME: FIXME! FIXME! Truncated reply handling! */
+	    rmsgappend(SS, 1, "\r   TRUNCATED REPLY!");
 	  }
 
 	  if (SS->verboselog)
 	    fprintf(SS->verboselog,"  left-over ANCOUNT=%d != 0! TC=%d\n",
 		    ancount, hp->tc);
 
+	  rmsgappend(SS, 1, "\r   AnswerCount  %d > 0!!", ancount);
+
+
 	  return EX_TEMPFAIL; /* FIXME?? FIXME?? */
 	}
 
@@ -359,7 +387,7 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    struct addrinfo *ai;
 	    Usockaddr *usa;
 	    char *canon;
-	    int   nlen = strlen(buf);
+	    int   nlen = strlen((const char*)buf);
 
 	    /* Pick the address data */
 	    for (i = 0; i < nmx; ++i) {
@@ -373,7 +401,10 @@ getmxrr(SS, host, mx, maxmx, depth)
 
 		/* WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!
 		   This assumes intimate knowledge of the system
-		   implementation of the  ``struct addrinfo'' !  */
+		   implementation of the  ``struct addrinfo'' !
+
+		   Linux and FreeBSD have DIFFERENT implementatoin!
+		   Other operating systems are likely different as well! */
 
 		if (SS->verboselog)
 		  fprintf(SS->verboselog,"  host='%s' AF=%s TTL=%d\n",
@@ -393,7 +424,7 @@ getmxrr(SS, host, mx, maxmx, depth)
 		ai->ai_protocol = IPPROTO_TCP;
 		ai->ai_addr     = (struct sockaddr *)usa;
 		ai->ai_addrlen  = sizeof(*usa);
-		ai->ai_canonname = canon;
+		ai->ai_canonname = /* canon */ NULL;
 
 		ai->ai_next     = mx[i].ai;
 		mx[i].ai        = ai;
@@ -465,13 +496,16 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    /* This resolves CNAME, it should not happen in case
 	       of MX server, though..    */
 #ifdef HAVE_GETADDRINFO
-	    n = getaddrinfo(mx[i].host, "0", &req, &ai);
+	    n = getaddrinfo((const char*)mx[i].host, "0", &req, &ai);
 #else
-	    n = _getaddrinfo_(mx[i].host, "0", &req, &ai, SS->verboselog);
+	    n = _getaddrinfo_((const char*)mx[i].host, "0", &req, &ai, SS->verboselog);
 #endif
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog,"  getaddrinfo('%s','0') (PF_INET) -> r=%d (%s), ai=%p\n",
 		      mx[i].host, n, gai_strerror(n), ai);
+	    if (n != 0)
+	      rmsgappend(SS, 1, "\r-> getaddrinfo(INET, '%s','0') -> r=%d (%s); ai=%p",
+			 mx[i].host, n, gai_strerror(n), ai);
 #if 0
 	    if (n) {
 	      zsyslog((LOG_INFO,"getmxrr('%s') mx[%d]='%s' getaddrinfo(INET) rc=%d",
@@ -513,15 +547,18 @@ getmxrr(SS, host, mx, maxmx, depth)
 	  /* This resolves CNAME, it should not happen in case
 	     of MX server, though..    */
 #ifdef HAVE_GETADDRINFO
-	    n2 = getaddrinfo(mx[i].host, "0", &req, &ai2);
+	    n2 = getaddrinfo((const char *)mx[i].host, "0", &req, &ai2);
 #else
-	    n2 = _getaddrinfo_(mx[i].host, "0", &req, &ai2,
+	    n2 = _getaddrinfo_((const char *)mx[i].host, "0", &req, &ai2,
 			       SS->verboselog);
 #endif
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog,"  getaddrinfo('%s','0') (PF_INET6) -> r=%d (%s), ai=%p\n",
 		      mx[i].host, n2, gai_strerror(n2), ai2);
 
+	    if (n2 != 0)
+	      rmsgappend(SS, 1, "\r-> getaddrinfo(INET6,'%s') -> r=%d (%s); ai=%p",
+			 mx[i].host, n2, gai_strerror(n2), ai2);
 #if 0
 	    if (n) {
 	      zsyslog((LOG_INFO,"getmxrr('%s') mx[%d]='%s' getaddrinfo(INET6) rc=%d",
@@ -568,8 +605,10 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    if (n == EAI_AGAIN) {
 	      sprintf(SS->remotemsg, "smtp; 500 (DNS: getaddrinfo<%.200s> got EAI_AGAIN)", buf);
 	      endtime = now;
+#ifndef TEST
 	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"4.4.4 (DNS lookup report)",SS->remotemsg);
+#endif
 
 	      had_eai_again = 1;
 	    }
@@ -587,7 +626,7 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    mx[nmx].ai = ai;
 	    ai         = ai->ai_next;
 	    mx[nmx].ai->ai_next = NULL;
-	    mx[nmx].host = (msgdata *)strdup(mx[i].host);
+	    mx[nmx].host = (msgdata *)strdup((const char *)mx[i].host);
 	    if (mx[nmx].host == NULL) {
 	      fprintf(stderr, "Out of virtual memory!\n");
 	      exit(EX_OSERR);
@@ -604,15 +643,22 @@ getmxrr(SS, host, mx, maxmx, depth)
 
 	for (i = 0; i < nmx; ++i) {
 	  if (mx[i].ai == NULL && mx[i].host != NULL) {
+	    rmsgappend(SS, 1, "\r-> No addresses for '%s'[%d]", mx[i].host, i);
 	    free(mx[i].host);
 	    mx[i].host = NULL;
 	    continue;
 	  }
-	  if (CISTREQ(mx[i].ai->ai_canonname, myhostname) ||
+	  if (CISTREQ(mx[i].host, myhostname) ||
+	      (mx[i].ai->ai_canonname &&
+	       CISTREQ(mx[i].ai->ai_canonname, myhostname)) ||
 	      matchmyaddresses(mx[i].ai) == 1) {
 
+	    rmsgappend(SS, 1, "\r-> Selfmatch '%s'(%s)[%d]",
+		       mx[i].host, ((mx[i].ai->ai_canonname) ?
+				    mx[i].ai->ai_canonname : "-"), i);
+
 	    if (SS->verboselog)
-	      fprintf(SS->verboselog,"  matchmyaddresses(): matched!  canon='%s', myname='%s'\n", mx[i].ai->ai_canonname, myhostname);
+	      fprintf(SS->verboselog,"  matchmyaddresses(): matched!  canon='%s', myname='%s'\n", mx[i].ai->ai_canonname ? mx[i].ai->ai_canonname : "<NIL>", myhostname);
 	    if (maxpref > (int)mx[i].pref)
 	      maxpref = mx[i].pref;
 	  }
@@ -622,6 +668,8 @@ getmxrr(SS, host, mx, maxmx, depth)
 
 	if (SS->verboselog)
 	  fprintf(SS->verboselog,"  getmxrr('%s') -> nmx=%d, maxpref=%d, realname='%s'\n", host, nmx, maxpref, realname);
+	rmsgappend(SS, 1, "\r-> nmx=%d maxpref=%d realname='%s'",
+		   nmx, maxpref, realname);
 
 	/* discard MX RRs with a value >= that of  myhost */
 	for (n = i = 0; n < nmx; ++n) {
@@ -633,13 +681,15 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    ++i; /* discard count */
 	  }
 	}
-	if (i == nmx) {	/* All discarded, we are the best MX :-( */
+	if (i /* discard count */ == nmx) {
+	  /* All discarded, we are the best MX :-( */
 	  mx[0].host = NULL;
 	  SS->mxcount = 0;
 	  if (had_eai_again)
 	    return EX_TEMPFAIL;
 	  return EX_OK;
 	}
+#ifndef TEST
 #ifdef	RFC974
 	/* discard MX's that do not support SMTP service */
 	if (checkwks)
@@ -659,6 +709,7 @@ getmxrr(SS, host, mx, maxmx, depth)
 	    }
 	  }
 #endif	/* RFC974 */
+#endif /* TEST */
 	/* determine how many are left */
 	for (i = 0, n = 0; i < nmx; ++i) {
 	  if (mx[i].host == NULL)
@@ -673,11 +724,12 @@ getmxrr(SS, host, mx, maxmx, depth)
 	  return EX_TEMPFAIL;
 
 	if (n == 0) {/* MX's exist, but their WKS's show no TCP smtp service */
-	  sprintf(SS->remotemsg,
-		  "smtp; 500 (DNS: MX host does not support SMTP: %.200s)", host);
+	  rmsgappend(SS, 1, "\r=> NONE of MXes support SMTP!");
 	  time(&endtime);
+#ifndef TEST
 	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",SS->remotemsg);
+#endif
 	  return EX_UNAVAILABLE;
 	}
 	nmx = n;
@@ -753,6 +805,94 @@ getmxrr(SS, host, mx, maxmx, depth)
 }
 
 
+#ifdef HAVE_STDARG_H
+#ifdef __STDC__
+void
+rmsgappend(SmtpState *SS, int append, const char *fmt, ...)
+#else /* Not ANSI-C */
+void
+rmsgappend(SS, append, fmt)
+	SmtpState *SS;
+	int append;
+	const char *fmt;
+#endif
+#else
+void
+rmsgappend(va_alist)
+	va_dcl
+#endif
+{
+	va_list ap;
+	char *args;
+	long  argi;
+	char *cp, *cpend;
+	char ibuf[15];
+	int  long_flg = 0;
+#ifdef HAVE_STDARG_H
+	va_start(ap,fmt);
+#else
+	const char *fmt;
+	SmtpState *SS;
+	int append;
+	va_start(ap);
+	SS     = va_arg(ap, SmtpState *);
+	append = va_arg(ap, int);
+	fmt    = va_arg(ap, char *);
+#endif
+
+	cp    = SS->remotemsg + strlen(SS->remotemsg);
+	cpend = SS->remotemsg + sizeof(SS->remotemsg) -1;
+
+	if (SS->prevcmdstate >= SMTPSTATE99) /* magic limit.. */
+	  SS->remotemsgs[(int)SS->cmdstate] = cp = SS->remotemsg;
+	if (SS->cmdstate > SS->prevcmdstate)
+	  SS->remotemsgs[(int)SS->cmdstate] = cp;
+
+	if (!append)
+	  cp = SS->remotemsgs[(int)SS->cmdstate];
+
+	SS->prevcmdstate = SS->cmdstate;
+
+	if (!fmt) fmt="(NULL)";
+	for (; *fmt != 0; ++fmt) {
+	  if (*fmt == '%') {
+	    int c = *++fmt;
+	    long_flg = 0;
+	    if (c == 'l') {
+	      long_flg = 1;
+	      c = *++fmt;
+	    }
+	    switch (c) {
+	    case 's':
+	      args = va_arg(ap, char *);
+	      if (!args) args = "(null)";
+	      while (*args && cp < cpend) *cp++ = *args ++;
+	      break;
+	    case 'd':
+	      if (long_flg) {
+		argi = va_arg(ap, long);
+		sprintf(ibuf, "%ld", argi);
+	      } else {
+		argi = va_arg(ap, int);
+		sprintf(ibuf, "%d", (int)argi);
+	      }
+	      args = ibuf;
+	      while (*args && cp < cpend) *cp++ = *args ++;
+	      break;
+	    default:
+	      if (cp < cpend) *cp++ = '%';
+	      if (cp < cpend) *cp++ = c;
+	      break;
+	    }
+	  } else
+	    if (cp < cpend)
+	      *cp++ = *fmt;
+	}
+	*cp = 0;
+	va_end(ap);
+}
+
+
 #ifdef TEST
 
 time_t endtime, starttime, now;
@@ -762,7 +902,7 @@ int use_ipv6 = 1;
 int prefer_ip6 = 1;
 int checkwks = 0;
 
-char myhostname[MAXHOSTNAMELEN+1] = "my.host.name";
+char myhostname[512] = "my.host.name";
 const char *progname;
 char errormsg[ZBUFSIZ]; /* Global for the use of  dnsgetrr.c */
 
@@ -909,4 +1049,23 @@ int main(argc, argv)
 
 	return 0;
 }
+
+const char * getzenv(name) 
+     const char *name;
+{
+   return NULL;
+}
+
+#if 0 /* Deep testing .. */
+int _getaddrinfo_ (host, port, req, ai, logfp)
+  const char *host;
+  const char *port;
+  const struct addrinfo *req;
+  struct addrinfo **ai;
+  FILE *logfp;
+{
+  return getaddrinfo(host, port, req, ai);
+}
+#endif
+
 #endif
