@@ -109,7 +109,7 @@ void mail_queue_path(buf, subdir, filename)
      char *subdir;
      char *filename;
 {
-  char *po = getzenv("POSTOFFICE");
+  const char *po = getzenv("POSTOFFICE");
   if (!po) po = POSTOFFICE;
 
   sprintf(buf, "%s/%s/%s", po, subdir, filename);
@@ -603,20 +603,21 @@ int     tls_init_clientengine(SS, cfgpath)
     if (s) *s = 0;
     s = (void*) buf;
 
-#define SKIPWHILE(X,Y)  while (*Y != '\0' && isascii(*Y) && X(*Y)) { ++Y; }
+#define SKIPSPACE(Y) while (*Y == ' ' || *Y == '\t' || *Y == '\n') ++Y
+#define SKIPTEXT(Y ) while (*Y && !(*Y == ' ' || *Y == '\t' || *Y == '\n')) ++Y
 
-    SKIPWHILE(isspace, s);
+    SKIPSPACE(s);
     if (!*s || *s == '#' || *s == ';')
       continue; /* First non-whitespace char is comment start (or EOL) */
 
-    SKIPWHILE( isspace, s);
-    n = s;
-    SKIPWHILE(!isspace, s);
+    SKIPSPACE(s);
+    n = (char *)s;
+    SKIPTEXT(s);
     if (*s) *s++ = 0;
 
-    SKIPWHILE( isspace, s);
-    a1 = s;
-    SKIPWHILE(!isspace, s);
+    SKIPSPACE(s);
+    a1 = (char *)s;
+    SKIPTEXT(s);
     if (*s) *s++ = 0;
 
 
@@ -839,7 +840,7 @@ int     tls_start_clienttls(SS,peername)
      * we can later check, whether it is really reused.
      */
     if (tls_use_scache) {
-	MD5(peername, strlen(peername), peername_md5);
+	MD5((void *)peername, strlen(peername), peername_md5);
 	session = load_clnt_session(peername_md5, MD5_DIGEST_LENGTH,
 				    &verify_result);
 	if (session) {
